package com.payments.core.sdktest.activities;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.NumberPicker;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.mikepenz.fontawesome_typeface_library.FontAwesome;
import com.mikepenz.iconics.IconicsDrawable;
import com.payments.core.CoreKeyedSecureCard;
import com.payments.core.CoreSale;
import com.payments.core.CoreSaleKeyed;
import com.payments.core.CoreSecureCard;
import com.payments.core.CoreTokenMethod;
import com.payments.core.CoreUnreferencedRefund;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.common.enums.CoreCardEntryMode;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.SignatureCollection;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.adapter.SimpleSpinnerAdapter;
import com.payments.core.sdktest.common.SimplePojo;
import com.payments.core.sdktest.common.UiHelper;
import com.payments.core.sdktest.common.Util;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

public class CardDetailsFragment extends Fragment implements View.OnClickListener, View.OnFocusChangeListener {
	private static final String ARGUMENT_IS_VIEWTYPE = "ARGUMENT_IS_VIEWTYPE";
	private static final String ARGUMENT_OBJECT_CORE = "ARGUMENT_OBJECT_CORE";
	private static final String ARGUMENT_IS_SECURE_CARD = "ARGUMENT_IS_SECURE_CARD";
	private static final String ARGUMENT_IS_MERCHANT_REFERENCE = "ARGUMENT_IS_MERCHANT_REFERENCE";
	private static final String ARGUMENT_IS_CARD_REFERENCE = "ARGUMENT_IS_CARD_REFERENCE";
	private static final String ARGUMENT_OBJECT_CORE_SECURE_CARD = "ARGUMENT_OBJECT_CORE_SECURE_CARD";
	private Button buttonSubmit;

	private ImageView iconCardNumber, iconName, iconCVV, iconReason;
	private Spinner spinnerCardType;
	private EditText editCardNumber, editCardName, editCVV, editReason, editCustomName1, editCustomName2, editCustomName3, editCustomName4,
			editCustomValue1, editCustomValue2, editCustomValue3, editCustomValue4, permittedTerminals, editEmail, editPhone;
	private NumberPicker pickerExpiryMonth, pickerExpiryYear;
	private TextView labelCardNumber, labelName, labelCVV, labelReason;
	private LinearLayout layout_custom_fields, topCardLayout;

	private SimpleSpinnerAdapter cardBrandListAdapter;
	private List<SimplePojo> cardBrandList;

	private CoreSale coreSale;
	private CoreUnreferencedRefund coreUnreferencedRefund;
	private ViewType viewType;
	private String merchantReference;
	private String cardReference;
	private SecureCardType secureCardType;
	private CoreSecureCard secureCard;

	public CardDetailsFragment() {
		// Required empty public constructor
	}

	public enum ViewType {
		SALE,
		REFUND,
		SECURE_CARD
	}

	public enum SecureCardType {
		REGISTER,
		UPDATE,
		SALE
	}

	public static CardDetailsFragment getInstance(ViewType vieType,
												  CoreSale coreSale,
												  CoreUnreferencedRefund coreUnreferencedRefund,
												  String merchantReference,
												  String cardReference,
												  SecureCardType secureCardType,
												  CoreSecureCard secureCard) {
		CardDetailsFragment cardDetailsFragment = new CardDetailsFragment();

		Bundle bundle = new Bundle();
		bundle.putSerializable(ARGUMENT_IS_VIEWTYPE, vieType);
		if (vieType.equals(ViewType.SECURE_CARD)) {
			bundle.putString(ARGUMENT_IS_MERCHANT_REFERENCE, merchantReference);
			bundle.putString(ARGUMENT_IS_CARD_REFERENCE, cardReference);
			bundle.putSerializable(ARGUMENT_IS_SECURE_CARD, secureCardType);
			bundle.putSerializable(ARGUMENT_OBJECT_CORE, coreSale);
			bundle.putSerializable(ARGUMENT_OBJECT_CORE_SECURE_CARD, secureCard);
		} else if (vieType.equals(ViewType.REFUND)) {
			bundle.putSerializable(ARGUMENT_OBJECT_CORE, coreUnreferencedRefund);
		} else {
			bundle.putSerializable(ARGUMENT_OBJECT_CORE, coreSale);
		}

		cardDetailsFragment.setArguments(bundle);

		return cardDetailsFragment;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		viewType = (ViewType) getArguments().get(ARGUMENT_IS_VIEWTYPE);
		if (viewType == null) {
			viewType = ViewType.SALE;
		}

		if (viewType.equals(ViewType.SECURE_CARD)) {
			merchantReference = getArguments().getString(ARGUMENT_IS_MERCHANT_REFERENCE);
			cardReference = getArguments().getString(ARGUMENT_IS_CARD_REFERENCE);
			secureCardType = (SecureCardType) getArguments().get(ARGUMENT_IS_SECURE_CARD);
			coreSale = (CoreSale) getArguments().getSerializable(ARGUMENT_OBJECT_CORE);
			secureCard = (CoreSecureCard) getArguments().getSerializable(ARGUMENT_OBJECT_CORE_SECURE_CARD);
		} else if (viewType.equals(ViewType.REFUND)) {
			coreUnreferencedRefund = (CoreUnreferencedRefund) getArguments().getSerializable(ARGUMENT_OBJECT_CORE);
		} else {
			coreSale = (CoreSale) getArguments().getSerializable(ARGUMENT_OBJECT_CORE);
		}
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_card_details, container, false);
		layout_custom_fields = (LinearLayout) view.findViewById(R.id.layout_custom_fields);
		layout_custom_fields.setVisibility(View.GONE);
		topCardLayout = (LinearLayout) view.findViewById(R.id.topCardLayout);
		topCardLayout.setVisibility(View.VISIBLE);
		editCustomName1 = (EditText) view.findViewById(R.id.custom_edit_name1);
		editCustomName2 = (EditText) view.findViewById(R.id.custom_edit_name2);
		editCustomName3 = (EditText) view.findViewById(R.id.custom_edit_name3);
		editCustomName4 = (EditText) view.findViewById(R.id.custom_edit_name4);
		editCustomValue1 = (EditText) view.findViewById(R.id.custom_edit_value1);
		editCustomValue2 = (EditText) view.findViewById(R.id.custom_edit_value2);
		editCustomValue3 = (EditText) view.findViewById(R.id.custom_edit_value3);
		editCustomValue4 = (EditText) view.findViewById(R.id.custom_edit_value4);
		permittedTerminals = (EditText) view.findViewById(R.id.permitted_terminals);
		editEmail = (EditText) view.findViewById(R.id.email);
		editPhone = (EditText) view.findViewById(R.id.phone);
		editCardNumber = (EditText) view.findViewById(R.id.card_details_edit_number);
		editCardName = (EditText) view.findViewById(R.id.card_details_edit_name);
		editCVV = (EditText) view.findViewById(R.id.card_details_edit_cvv);
		editReason = (EditText) view.findViewById(R.id.card_details_edit_reason);
		pickerExpiryMonth = (NumberPicker) view.findViewById(R.id.card_details_expiry_month);
		pickerExpiryYear = (NumberPicker) view.findViewById(R.id.card_details_expiry_year);
		iconCardNumber = (ImageView) view.findViewById(R.id.card_details_icon_number);
		iconName = (ImageView) view.findViewById(R.id.card_details_icon_name);
		iconCVV = (ImageView) view.findViewById(R.id.card_details_icon_cvv);
		iconReason = (ImageView) view.findViewById(R.id.card_details_icon_reason);
		spinnerCardType = (Spinner) view.findViewById(R.id.card_details_spinner_brand);
		labelCardNumber = (TextView) view.findViewById(R.id.card_details_label_card_number);
		labelName = (TextView) view.findViewById(R.id.card_details_label_name);
		labelCVV = (TextView) view.findViewById(R.id.card_details_label_card_cvv);
		labelReason = (TextView) view.findViewById(R.id.card_details_label_reason);
		buttonSubmit = (Button) view.findViewById(R.id.card_details_button_submit);
		buttonSubmit.setBackgroundColor(getResources().getColor(R.color.colorPrimary));

		if (viewType.equals(ViewType.SECURE_CARD)) {
			layout_custom_fields.setVisibility(View.VISIBLE);
			if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) {
				topCardLayout.setVisibility(View.GONE);
			}
		}
		iconCardNumber.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_credit_card).color(getResources().getColor(R.color.colorPrimary)).sizeDp(25));
		iconName.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_user).color(getResources().getColor(R.color.colorPrimary)).sizeDp(25));
		iconCVV.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_lock).color(getResources().getColor(R.color.colorPrimary)).sizeDp(25));
		iconReason.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_comment).color(getResources().getColor(R.color.colorPrimary)).sizeDp(25));

		buttonSubmit.setOnClickListener(this);

		pickerExpiryMonth.setMinValue(1);
		pickerExpiryMonth.setMaxValue(12);
		pickerExpiryMonth.setValue(Calendar.getInstance().get(Calendar.MONTH) + 1);
		pickerExpiryMonth.setFormatter(new NumberPicker.Formatter() {
			@Override
			public String format(int i) {
				return String.format("%02d", i);
			}
		});

		pickerExpiryYear.setMinValue(Calendar.getInstance().get(Calendar.YEAR));
		pickerExpiryYear.setMaxValue(2099);
		pickerExpiryYear.setValue(Calendar.getInstance().get(Calendar.YEAR));

		editCardNumber.setOnFocusChangeListener(this);
		editCardName.setOnFocusChangeListener(this);
		editCVV.setOnFocusChangeListener(this);
		editReason.setOnFocusChangeListener(this);

		editCardNumber.addTextChangedListener(Util.addCardNumberMask());

		cardBrandList = new ArrayList<>();
		for (String brand : getResources().getStringArray(R.array.supported_card_brands)) {
			Long brandID = Long.parseLong(brand.split(":")[0]);
			String brandDescription = brand.split(":")[1];

			cardBrandList.add(new SimplePojo(brandDescription, brandID));
		}

		cardBrandListAdapter = new SimpleSpinnerAdapter(getContext(), cardBrandList);
		spinnerCardType.setAdapter(cardBrandListAdapter);

		if (viewType.equals(ViewType.REFUND)) {
			labelReason.setVisibility(View.VISIBLE);
			editReason.setVisibility(View.VISIBLE);
			iconReason.setVisibility(View.VISIBLE);
		} else {
			labelReason.setVisibility(View.GONE);
			editReason.setVisibility(View.GONE);
			iconReason.setVisibility(View.GONE);
		}

		if (secureCard != null) {
			if (secureCard.getCoreTokenMethod() != null && secureCard.getCoreTokenMethod().getKeyedSecureCard() != null) {
				CoreKeyedSecureCard coreKeyedSecureCard = secureCard.getCoreTokenMethod().getKeyedSecureCard();
				editCVV.setText(coreKeyedSecureCard.getCvv());
				editCardName.setText(coreKeyedSecureCard.getCardHolderName());
				editCardNumber.setText(coreKeyedSecureCard.getCardNumber());
				char[] expiryDate = coreKeyedSecureCard.getExpiryDate().toCharArray();
				int month = Integer.parseInt(String.valueOf(expiryDate[0]) + String.valueOf(expiryDate[1]));
				int year = Integer.parseInt("20" + String.valueOf(expiryDate[2]) + String.valueOf(expiryDate[3]));

				if (month < pickerExpiryMonth.getMinValue()) {
					pickerExpiryMonth.setValue(pickerExpiryMonth.getMinValue());
				} else if (month > pickerExpiryMonth.getMaxValue()) {
					pickerExpiryMonth.setValue(pickerExpiryMonth.getMaxValue());
				} else {
					pickerExpiryMonth.setValue(month);
				}

				if (year < pickerExpiryYear.getMinValue()) {
					pickerExpiryYear.setValue(pickerExpiryYear.getMinValue());
				} else if (year > pickerExpiryYear.getMaxValue()) {
					pickerExpiryYear.setValue(pickerExpiryYear.getMaxValue());
				} else {
					pickerExpiryYear.setValue(year);
				}
			}

			if (secureCard.getCustomFields() != null) {
				List<CoreSecureCard.CoreCustomField> listOfCustomFields = secureCard.getCustomFields();

				editCustomName1.setText(listOfCustomFields.get(0).getName());
				editCustomValue1.setText(listOfCustomFields.get(0).getValue());

				if (secureCard.getCustomFields().size() > 1) {
					editCustomName2.setText(listOfCustomFields.get(1).getName());
					editCustomValue2.setText(listOfCustomFields.get(1).getValue());
				}

				if (secureCard.getCustomFields().size() > 2) {
					editCustomName3.setText(listOfCustomFields.get(2).getName());
					editCustomValue3.setText(listOfCustomFields.get(2).getValue());
				}

				if (secureCard.getCustomFields().size() > 3) {
					editCustomName4.setText(listOfCustomFields.get(3).getName());
					editCustomValue4.setText(listOfCustomFields.get(3).getValue());
				}
			}

			permittedTerminals.setText(secureCard.getPermittedTerminals());
			editEmail.setText(secureCard.getEmail());
			editPhone.setText(secureCard.getMobileNumber());
		}

		return view;
	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.card_details_button_submit:

				if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) {
					if (editCardNumber.getText().toString().isEmpty() || editCardNumber.getText().length() > 20) {
						UiHelper.getInstance().showError(getActivity(), "Error", getString(R.string.messages_app_invalid_card_number));
						return;
					}

					if (editCardName.getText().toString().isEmpty()) {
						UiHelper.getInstance().showError(getActivity(), "Error", getString(R.string.messages_app_invalid_name));
						return;
					}

					if (editCVV.getText().toString().isEmpty() || editCVV.getText().length() > 4) {
						UiHelper.getInstance().showError(getActivity(), "Error", getString(R.string.messages_app_invalid_card_cvv));
						return;
					}
				}

				if (viewType.equals(ViewType.REFUND)) {
					doUnreferencedRefund();
				} else if (viewType.equals(ViewType.SALE)) {
					doKeyedSale();
				} else {
					doSecureCard();
				}

				break;
		}
	}

	@Override
	public void onFocusChange(View view, boolean focused) {
		TextView label = labelCardNumber;

		switch (view.getId()) {
			case R.id.card_details_edit_number:
				label = labelCardNumber;

				break;
			case R.id.card_details_edit_name:
				label = labelName;

				break;
			case R.id.card_details_edit_cvv:
				label = labelCVV;

				break;
			case R.id.card_details_edit_reason:
				label = labelReason;

				break;
		}

		if (focused) {
			label.setScaleX(0.7F);
			label.setScaleY(0.7F);

		} else {
			if (((EditText) view).getText().toString().isEmpty()) {
				label.setScaleX(1);
				label.setScaleY(1);

			} else {
				label.setScaleX(0.7F);
				label.setScaleY(0.7F);
			}
		}
	}

	private void doUnreferencedRefund() {
		if (editReason.getText().toString().isEmpty()) {
			UiHelper.getInstance().showError(getActivity(), "Error", getString(R.string.reason_empty));
			return;
		}
		coreUnreferencedRefund.setCardHolderName(editCardName.getText().toString());
		coreUnreferencedRefund.setCardNumber(editCardNumber.getText().toString().replace(" ", ""));
		coreUnreferencedRefund.setCardCvv(editCVV.getText().toString());
		coreUnreferencedRefund.setCardType(((SimplePojo) spinnerCardType.getSelectedItem()).getDescription());
		coreUnreferencedRefund.setExpiryDate(String.format("%02d", pickerExpiryMonth.getValue()) + String.valueOf(pickerExpiryYear.getValue()).substring(2));
		coreUnreferencedRefund.setReason(editReason.getText().toString());
		AndroidTerminal.getInstance().processUnreferencedRefund(coreUnreferencedRefund);
		UiHelper.getInstance().showDialog("Loading...", getActivity(), false);
	}

	private void doKeyedSale() {
		CoreSaleKeyed saleKeyed = new CoreSaleKeyed(coreSale.getAmount(), coreSale.getOrderId());
		saleKeyed.setCardHolderName(editCardName.getText().toString());
		saleKeyed.setCardNumber(editCardNumber.getText().toString().replace(" ", ""));
		saleKeyed.setCardCvv(editCVV.getText().toString());
		saleKeyed.setCardType(((SimplePojo) spinnerCardType.getSelectedItem()).getDescription());
		saleKeyed.setExpiryDate(String.format("%02d", pickerExpiryMonth.getValue()) + String.valueOf(pickerExpiryYear.getValue()).substring(2));
		saleKeyed.addTax(coreSale.getTax());
		saleKeyed.setTip(coreSale.getTip());
		AndroidTerminal.getInstance().processSale(saleKeyed);
		UiHelper.getInstance().showDialog("Loading...", getActivity(), false);
	}

	private void doSecureCard() {
		CoreSecureCard coreSecureCard = new CoreSecureCard();

		if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) {
			if (secureCardType.equals(SecureCardType.UPDATE)) {
				if (secureCard.getCoreTokenMethod().getKeyedSecureCard() == null) {
					coreSecureCard.getCoreTokenMethod().setKeyedSecureCard(new CoreKeyedSecureCard());
				}
				if (!editCardNumber.getText().toString().contains("*")) {
					coreSecureCard.setCoreCardEntryMode(CoreCardEntryMode.KEYED);
				}
				coreSecureCard.setCoreTokenMethod(secureCard.getCoreTokenMethod());
			} else {
				coreSecureCard.setCoreTokenMethod(new CoreTokenMethod(new CoreKeyedSecureCard()));
				coreSecureCard.setCoreCardEntryMode(CoreCardEntryMode.KEYED);
			}

			coreSecureCard.getCoreTokenMethod().getKeyedSecureCard().setCvv(editCVV.getText().toString());
			coreSecureCard.getCoreTokenMethod().getKeyedSecureCard().setCardHolderName(editCardName.getText().toString());
			coreSecureCard.getCoreTokenMethod().getKeyedSecureCard().setCardNumber(editCardNumber.getText().toString().replace(" ", ""));
			coreSecureCard.getCoreTokenMethod().getKeyedSecureCard().setExpiryDate(String.format("%02d", pickerExpiryMonth.getValue()) + String.valueOf(pickerExpiryYear.getValue()).substring(2));
		}

		CoreSecureCard.CoreCustomField field;
		List<CoreSecureCard.CoreCustomField> customFields = null;

		if (!editCustomName1.getText().toString().trim().isEmpty()) {
			if (editCustomValue1.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 1", Toast.LENGTH_SHORT).show();
				return;
			}

			if (customFields == null) {
				customFields = new ArrayList<>();
			}

			field = new CoreSecureCard.CoreCustomField(editCustomName1.getText().toString(), editCustomValue1.getText().toString());
			customFields.add(field);
		}

		if (!editCustomName2.getText().toString().trim().isEmpty()) {
			if (editCustomValue2.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 2", Toast.LENGTH_SHORT).show();
				return;
			}

			if (customFields == null) {
				customFields = new ArrayList<>();
			}

			field = new CoreSecureCard.CoreCustomField(editCustomName2.getText().toString(), editCustomValue2.getText().toString());
			customFields.add(field);
		}

		if (!editCustomName3.getText().toString().trim().isEmpty()) {
			if (editCustomValue3.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 3", Toast.LENGTH_SHORT).show();
				return;
			}

			if (customFields == null) {
				customFields = new ArrayList<>();
			}

			field = new CoreSecureCard.CoreCustomField(editCustomName3.getText().toString(), editCustomValue3.getText().toString());
			customFields.add(field);
		}

		if (!editCustomName4.getText().toString().trim().isEmpty()) {
			if (editCustomValue4.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 4", Toast.LENGTH_SHORT).show();
				return;
			}

			if (customFields == null) {
				customFields = new ArrayList<>();
			}

			field = new CoreSecureCard.CoreCustomField(editCustomName4.getText().toString(), editCustomValue4.getText().toString());
			customFields.add(field);
		}

		if (secureCardType.equals(SecureCardType.SALE)) {
			if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) {
				coreSale.setCoreSecureCard(coreSecureCard);
				coreSale.getCoreSecureCard().setMerchantReference(merchantReference);
				coreSale.getCoreSecureCard().setEmail(editEmail.getText().toString());
				coreSale.getCoreSecureCard().setMobileNumber(editPhone.getText().toString());
				coreSale.getCoreSecureCard().setPermittedTerminals(permittedTerminals.getText().toString());
				coreSale.getCoreSecureCard().setCustomFields(customFields);
				coreSale.setSignatureCollection(SignatureCollection.MANUAL);
				AndroidTerminal.getInstance().processSale(coreSale);
			} else {
				CoreSaleKeyed coreSaleKeyed = new CoreSaleKeyed(coreSale.getAmount(), coreSale.getOrderId());
				coreSaleKeyed.setCardHolderName(editCardName.getText().toString());
				coreSaleKeyed.setCardNumber(editCardNumber.getText().toString().replace(" ", ""));
				coreSaleKeyed.setCardCvv(editCVV.getText().toString());
				coreSaleKeyed.setCardType(((SimplePojo) spinnerCardType.getSelectedItem()).getDescription());
				coreSaleKeyed.setExpiryDate(String.format("%02d", pickerExpiryMonth.getValue()) + String.valueOf(pickerExpiryYear.getValue()).substring(2));
				coreSaleKeyed.setCoreSecureCard(coreSecureCard);
				coreSaleKeyed.getCoreSecureCard().setMerchantReference(merchantReference);
				coreSaleKeyed.getCoreSecureCard().setEmail(editEmail.getText().toString());
				coreSaleKeyed.getCoreSecureCard().setMobileNumber(editPhone.getText().toString());
				coreSaleKeyed.getCoreSecureCard().setPermittedTerminals(permittedTerminals.getText().toString());
				coreSaleKeyed.getCoreSecureCard().setCustomFields(customFields);
				AndroidTerminal.getInstance().processSale(coreSaleKeyed);
			}
		} else {
			coreSecureCard.setMerchantReference(merchantReference);
			coreSecureCard.setCardReference(cardReference);
			coreSecureCard.setPermittedTerminals(permittedTerminals.getText().toString());
			coreSecureCard.setEmail(editEmail.getText().toString());
			coreSecureCard.setMobileNumber(editPhone.getText().toString());
			coreSecureCard.setCustomFields(customFields);

			if (secureCardType.equals(SecureCardType.REGISTER)) {
				AndroidTerminal.getInstance().registerSecureCard(coreSecureCard);
			} else {
				AndroidTerminal.getInstance().editSecureCard(coreSecureCard);
			}
		}
		UiHelper.getInstance().showDialog("Loading...", getActivity(), false);
	}
}
