package com.payments.core.sdktest.activities;

import android.Manifest;
import android.annotation.TargetApi;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.support.design.widget.CoordinatorLayout;
import android.support.design.widget.NavigationView;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.preference.PreferenceManager;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.security.ProviderInstaller;
import com.payments.core.CoreCloseBatchResponse;
import com.payments.core.CoreSettings;
import com.payments.core.CoreTransactionReports;
import com.payments.core.CoreTransactions;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.admin.rest.RestConnector;
import com.payments.core.common.contracts.CoreAPILogsListener;
import com.payments.core.common.contracts.CoreAPIReportingListener;
import com.payments.core.common.contracts.CoreAPISettingsListener;
import com.payments.core.common.enums.CoreError;
import com.payments.core.common.enums.CoreMode;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.LogLevel;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.common.UiHelper;

import java.lang.reflect.Constructor;
import java.util.ArrayList;

public class MainActivity extends AppCompatActivity implements CoreAPILogsListener, CoreAPISettingsListener, CoreAPIReportingListener, PaymentFragment.OnFragmentLoadedListener {
	protected static final int REQUEST_PERMISSIONS = 1;
	private NavigationView navigationView;
	private DrawerLayout drawer;
	private Button drawerButton;
	private Toolbar toolbar;
	private FrameLayout frameContent;
	public static int navItemIndex = 0;
	private static final String TAG_PAYMENT_SCREEN = "payments";
	private static final String TAG_REFUND_SCREEN = "refund";
	private static final String TAG_SETTINGS = "settings";
	private static final String TAG_SECURE_SCREEN = "secure";
	private static final String TAG_TERMINAL_SETTINGS_SCREEN = "terminal_settings";
	private static final String TAG_SECURE_CARD_SCREEN = "secure_card";
	public static String CURRENT_TAG = TAG_PAYMENT_SCREEN;
	private String[] activityTitles;
	private Handler mHandler;
	private TextView toolbar_title, toolbar_mode;
	private Fragment currentFragment;
	private CoreSettings settings;
	private boolean isDrawerLocked;
	private PaymentFragment paymentFragment;
	private PaymentFragment refundFragment;
	private ReportingFragment reportingFragment;
	private TerminalSettingsFragment terminalSettingsFragment;
	private SecureCardFragment secureFragment;
	private SettingsFragment settingsFragment;
	private Context currentContext;
	private String currentTerminalId, currentSecret;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
			setContentView(R.layout.drawer_fixed);

		} else {
			setContentView(R.layout.drawer);
		}
		createAllFragments();

		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
			try {
				ProviderInstaller.installIfNeeded(this);
			} catch (Exception e) {
				Log.d("SDK EXAMPLE APP", "GooglePlayServicesRepairableException: " + e.getMessage());
			}
		}

		toolbar = (Toolbar) findViewById(R.id.toolbar_actionbar);
		setSupportActionBar(toolbar);
		mHandler = new Handler();
		drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
		navigationView = (NavigationView) findViewById(R.id.nav_view);
		activityTitles = getResources().getStringArray(R.array.drawer);
		toolbar_title = (TextView) findViewById(R.id.toolbar_title);
		toolbar_mode = (TextView) findViewById(R.id.toolbar_mode);
		drawerButton = (Button) findViewById(R.id.toolbarButton);
		frameContent = (FrameLayout) findViewById(R.id.content_frame);
		toolbar_mode.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				try {
					Class aClass = Class.forName("com.payments.core.sdktest.common.ChangeHostDialog");
					Constructor constructor = aClass.getConstructor(Context.class);
					constructor.newInstance(MainActivity.this);
				} catch (Exception ignored) {
				}
			}
		});
		setUpNavigationDrawer();
		AndroidTerminal.getInstance().initialize();
		AndroidTerminal.getInstance().registerCoreAPISettingsListener(this);
		AndroidTerminal.getInstance().setMode(CoreMode.TEST);
		AndroidTerminal.getInstance().setLogLevel(LogLevel.LEVEL_FULL);
		AndroidTerminal.getInstance().registerLogListener(this);
		AndroidTerminal.getInstance().registerCoreAPIReportingListener(this);
		ArrayList<String> deniedPermissions = checkPermissions();
		if (deniedPermissions.size() > 0) {
			ActivityCompat.requestPermissions(this, deniedPermissions.toArray(new String[0]), REQUEST_PERMISSIONS);
		} else {
			checkSettings();
		}
	}

	private void checkSettings() {
		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
		String terminalId = preferences.getString("terminalIdPrefs", "");
		String secret = preferences.getString("secretPrefs", "");
		String mode = preferences.getString("modeList", "2");
		String level = preferences.getString("logLevel", "1");
		CoreMode coreMode = CoreMode.TEST;
		switch (mode) {
			case "1":
				coreMode = CoreMode.LIVE;
				break;
			case "2":
				coreMode = CoreMode.TEST;
				break;
			case "3":
				coreMode = CoreMode.DEV;
				break;
		}

		LogLevel logLevel = LogLevel.LEVEL_FULL;
		switch (level) {
			case "1": // FULL
				logLevel = LogLevel.LEVEL_FULL;
				break;
			case "2": // INFO
				logLevel = LogLevel.LEVEL_INFO;
				break;
			case "3": // ERROR
				logLevel = LogLevel.LEVEL_ERROR;
				break;
			case "4": // NONE
				logLevel = LogLevel.LEVEL_NONE;
				break;
		}

		if (RestConnector.isPayconexSDK()) {
			getSettings(this, "", "", coreMode);
			return;
		}
		AndroidTerminal.getInstance().setLogLevel(logLevel);
		if (terminalId.isEmpty() || secret.isEmpty()) {
			navItemIndex = 0;
			CURRENT_TAG = TAG_PAYMENT_SCREEN;
			loadFragment();
			AndroidTerminal.getInstance().setMode(coreMode);
			toolbar_mode.setText(coreMode.name());
			UiHelper.getInstance().showDialog("Loading...", this, false);
			AndroidTerminal.getInstance().initWithConfiguration(this);
		} else {
			getSettings(this, terminalId, secret, coreMode);
		}
	}


	@Override
	public void onFragmentLoaded() {
		UiHelper.getInstance().showDialog("Loading...", this, false);
		if (currentTerminalId.isEmpty() || currentSecret.isEmpty()) {
			AndroidTerminal.getInstance().initWithConfiguration(this);
		} else {
			AndroidTerminal.getInstance().initWithConfiguration(currentContext, currentTerminalId, currentSecret);
		}
	}

	public void getSettings(Context context, String terminalId, String secret, CoreMode coreMode) {
		if (RestConnector.isPayconexSDK()) {
			setSettingsPayconex();
			terminalId = "220614969941";
			secret = "0e8d38a08f2d4919a5a27ec83a6e8d76";
		}
		currentContext = context;
		currentTerminalId = terminalId;
		currentSecret = secret;
		navItemIndex = 0;
		CURRENT_TAG = TAG_PAYMENT_SCREEN;
		loadFragment();
		AndroidTerminal.getInstance().setMode(coreMode);
		toolbar_mode.setText(coreMode.name());
		if (paymentFragment != null) {
			paymentFragment.getSettings();
		}
	}

	private void setSettingsPayconex() {
		navigationView.getMenu().findItem(R.id.nav_terminal_settings).setVisible(false);
		navigationView.getMenu().findItem(R.id.nav_secure_card).setVisible(false);
		navigationView.getMenu().findItem(R.id.nav_refund).setVisible(false);
	}

	private void loadFragment() {
		selectNavMenu();
		setToolbarTitle();
		Runnable mPendingRunnable = new Runnable() {
			@Override
			public void run() {
				currentFragment = getFragment();
				setColor(R.color.colorPrimary);
				FragmentTransaction fragmentTransaction = getSupportFragmentManager().beginTransaction();
				fragmentTransaction.setCustomAnimations(android.R.anim.fade_in,
						android.R.anim.fade_out);
				fragmentTransaction.replace(R.id.content_frame, currentFragment, CURRENT_TAG);
				fragmentTransaction.commitAllowingStateLoss();
			}
		};
		mHandler.post(mPendingRunnable);

		if (!isDrawerLocked) {
			drawer.closeDrawers();
		}

		invalidateOptionsMenu();
	}

	private Fragment getFragment() {
		switch (navItemIndex) {
			case 0: // payment
				paymentFragment.setViewType(PaymentFragment.ViewType.SALE);
				return paymentFragment;
			case 1: // refund
				refundFragment.setViewType(PaymentFragment.ViewType.REFUND);
				return refundFragment;
			case 2:
				// reporting
				return reportingFragment;
			case 3:
				// terminal settings
				terminalSettingsFragment.setSettings(settings);
				return terminalSettingsFragment;
			case 4:
				// secure card
				return secureFragment;
			case 5:
				// settings
				settingsFragment.setPaymentFragment(this);
				return settingsFragment;
			default:
				return new PaymentFragment();
		}
	}

	private void createAllFragments() {
		paymentFragment = new PaymentFragment();
		refundFragment = new PaymentFragment();
		reportingFragment = new ReportingFragment();
		terminalSettingsFragment = new TerminalSettingsFragment();
		secureFragment = new SecureCardFragment();
		settingsFragment = new SettingsFragment();
	}

	private void setColor(int id) {
		if (toolbar != null)
			toolbar.setBackgroundColor(getResources().getColor(id));
		setStatusBarColor();
	}

	@TargetApi(Build.VERSION_CODES.LOLLIPOP)
	private void setStatusBarColor() {
		if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
			Window window = this.getWindow();
			window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
			window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
			window.setStatusBarColor(ContextCompat.getColor(this, R.color.colorPrimaryDark));
		}
	}

	private void setToolbarTitle() {
		toolbar_title.setText(activityTitles[navItemIndex]);
	}

	private void selectNavMenu() {
		navigationView.getMenu().getItem(navItemIndex).setChecked(true);
	}

	private void setUpNavigationDrawer() {
		if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
			isDrawerLocked = true;

			drawerButton.setVisibility(View.GONE);
			drawer.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_OPEN);
			drawer.setScrimColor(getResources().getColor(R.color.drawer_no_shadow));

			drawer.postDelayed(new Runnable() {
				@Override
				public void run() {
					CoordinatorLayout.LayoutParams layoutParams = (CoordinatorLayout.LayoutParams) frameContent.getLayoutParams();
					layoutParams.setMargins(drawer.getWidth() / 4, 0, 0, 0);
					frameContent.setLayoutParams(layoutParams);
				}
			}, 500);
		} else {
			isDrawerLocked = false;

			drawer.postDelayed(new Runnable() {
				@Override
				public void run() {
					CoordinatorLayout.LayoutParams layoutParams = (CoordinatorLayout.LayoutParams) frameContent.getLayoutParams();
					layoutParams.setMargins(0, 0, 0, 0);
					frameContent.setLayoutParams(layoutParams);
				}
			}, 500);

			drawerButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					drawer.openDrawer(Gravity.LEFT);
				}
			});
		}

		navigationView.setNavigationItemSelectedListener(new NavigationView.OnNavigationItemSelectedListener() {
			// This method will trigger on item Click of navigation menu
			@Override
			public boolean onNavigationItemSelected(MenuItem menuItem) {
				switch (menuItem.getItemId()) {
					case R.id.nav_sale:
						navItemIndex = 0;
						CURRENT_TAG = TAG_PAYMENT_SCREEN;
						break;
					case R.id.nav_refund:
						navItemIndex = 1;
						CURRENT_TAG = TAG_REFUND_SCREEN;
						break;
					case R.id.nav_secure:
						navItemIndex = 2;
						CURRENT_TAG = TAG_SECURE_SCREEN;
						break;
					case R.id.nav_terminal_settings:
						navItemIndex = 3;
						CURRENT_TAG = TAG_TERMINAL_SETTINGS_SCREEN;
						break;
					case R.id.nav_secure_card:
						navItemIndex = 4;
						CURRENT_TAG = TAG_SECURE_CARD_SCREEN;
						break;
					case R.id.nav_settings:
						navItemIndex = 5;
						CURRENT_TAG = TAG_SETTINGS;
						break;
					default:
						navItemIndex = 0;
				}
				if (menuItem.isChecked()) {
					menuItem.setChecked(false);
				} else {
					menuItem.setChecked(true);
				}
				menuItem.setChecked(true);
				loadFragment();
				return true;
			}
		});
	}

	private ArrayList<String> checkPermissions() {
		int permissionLocation = ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION);
		int permissionMicrophone = ContextCompat.checkSelfPermission(this, Manifest.permission.RECORD_AUDIO);
		int permissionStorage = ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
		ArrayList<String> deniedPermissions = new ArrayList<>();

		if (permissionLocation != PackageManager.PERMISSION_GRANTED) {
			deniedPermissions.add(Manifest.permission.ACCESS_COARSE_LOCATION);
		}

		if (permissionMicrophone != PackageManager.PERMISSION_GRANTED) {
			deniedPermissions.add(Manifest.permission.RECORD_AUDIO);
		}

		if (permissionStorage != PackageManager.PERMISSION_GRANTED) {
			deniedPermissions.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
		}
		return deniedPermissions;
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		super.onRequestPermissionsResult(requestCode, permissions, grantResults);
		switch (requestCode) {
			case REQUEST_PERMISSIONS:
				if (grantResults.length <= 0 || grantResults[0] != PackageManager.PERMISSION_GRANTED) {
					Toast.makeText(this, "All the permissions are necessary. The application may not work properly.", Toast.LENGTH_LONG).show();
				} else {
					checkSettings();
				}
		}
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		AndroidTerminal.getInstance().releaseResources();
		System.exit(0); //kill the app
	}

	@Override
	protected void onNewIntent(Intent intent) {
		super.onNewIntent(intent);
		setIntent(intent);
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (currentFragment instanceof PaymentFragment) {
			((PaymentFragment) currentFragment).onActivityResult(requestCode, resultCode, data);
		}
		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onBackPressed() {
		if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)){
			return;
		}
		if (drawer.isDrawerOpen(GravityCompat.START) && !isDrawerLocked) {
			drawer.closeDrawers();
			return;
		}
		super.onBackPressed();
	}

	@Override
	public void onSettingsRetrieved(CoreSettings settings) {
		Log.d("MainActivity", "onSettingsRetrieved ");
		this.settings = settings;
		if (currentFragment instanceof PaymentFragment) {
			((PaymentFragment) currentFragment).onSettingsRetrieved(settings);
		} else if (currentFragment instanceof TerminalSettingsFragment) {
			((TerminalSettingsFragment) currentFragment).onSettingsRetrieved(settings);
		}
		UiHelper.getInstance().dismissProgress();
	}

	@Override
	public void onError(CoreError error, String message) {
		Log.d("MainActivity", "onError " + error.name());
		UiHelper.getInstance().showError(this, error.name().replace("_", ""), message);
		if (error.equals(CoreError.INCORRECT_SETTINGS) || error.equals(CoreError.INCORRECT_SETTINGS_SECRET) || error.equals(CoreError.INCORRECT_SETTINGS_TERMINAL) || error.equals(CoreError.TERMINALID_OR_SECRET_NOT_DEFINED_IN_PAYCONFIG)) {
			SettingsFragment settingsFragment = new SettingsFragment();
			settingsFragment.setPaymentFragment(this);
			UiHelper.getInstance().settingsMissing(this, settingsFragment, getSupportFragmentManager());
		}
	}

	@Override
	public void onLogMessage(String message) {


	}

	@Override
	public void onTransactionListResponse(CoreTransactions response) {

	}

	@Override
	public void onRequestCloseBatchResponse(CoreCloseBatchResponse response) {
		UiHelper.getInstance().showSuccess(this, "New Transaction Batch Time", response.getBatchTime());
	}

	@Override
	public void onLoginUrlRetrieved(String url) {

	}

	@Override
	public void onTransactionReportRetrieved(CoreTransactionReports response) {

	}

	@Override
	public boolean onKeyUp(int keyCode, KeyEvent event) {
		if (currentFragment instanceof PaymentFragment) {
			((PaymentFragment) currentFragment).onKeyUp(keyCode, event);
		}
		return super.onKeyUp(keyCode, event);
	}
}