package com.payments.core.sdktest.activities;


import android.app.AlertDialog;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.text.InputType;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.payments.core.CoreAvailableBalance;
import com.payments.core.CoreBalanceInquiryEbt;
import com.payments.core.CoreCloseBatchResponse;
import com.payments.core.CoreCustomField;
import com.payments.core.CoreLoyaltyCardData;
import com.payments.core.CoreRefundResponse;
import com.payments.core.CoreResponse;
import com.payments.core.CoreSale;
import com.payments.core.CoreSaleEbt;
import com.payments.core.CoreSaleResponse;
import com.payments.core.CoreSetting;
import com.payments.core.CoreSettings;
import com.payments.core.CoreSignature;
import com.payments.core.CoreTax;
import com.payments.core.CoreTerminal;
import com.payments.core.CoreTip;
import com.payments.core.CoreTransactionReportSummary;
import com.payments.core.CoreTransactionReports;
import com.payments.core.CoreTransactions;
import com.payments.core.CoreUnreferencedRefund;
import com.payments.core.CoreUnreferencedRefundEbt;
import com.payments.core.CoreUtil;
import com.payments.core.CoreVasSmartTapConfig;
import com.payments.core.CoreVoucher;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.admin.rest.RestConnector;
import com.payments.core.common.contracts.CoreAPIDeviceListener;
import com.payments.core.common.contracts.CoreAPIMessageListener;
import com.payments.core.common.contracts.CoreAPIRefundListener;
import com.payments.core.common.contracts.CoreAPIReportingListener;
import com.payments.core.common.contracts.CoreAPIReversalListener;
import com.payments.core.common.contracts.CoreAPISaleListener;
import com.payments.core.common.contracts.CoreAPIUpdateListener;
import com.payments.core.common.enums.CardAccount;
import com.payments.core.common.enums.CoreDeviceError;
import com.payments.core.common.enums.CoreEmvType;
import com.payments.core.common.enums.CoreError;
import com.payments.core.common.enums.CoreMessage;
import com.payments.core.common.enums.CoreTransactionInputMethod;
import com.payments.core.common.enums.Currency;
import com.payments.core.common.enums.DebitAccountType;
import com.payments.core.common.enums.DeviceConnectionType;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.RequestPinType;
import com.payments.core.common.enums.SaleRequestType;
import com.payments.core.common.enums.SignatureCollection;
import com.payments.core.common.enums.TerminalType;
import com.payments.core.common.enums.TipType;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.common.AndroidPay;
import com.payments.core.sdktest.common.InputAccountDialog;
import com.payments.core.sdktest.common.UiHelper;
import com.payments.core.sdktest.common.Util;
import com.payments.core.sdktest.view.CanvasView;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class PaymentFragment extends Fragment implements Util.OnAmountChangeListener, CoreAPIDeviceListener, CoreAPISaleListener, CoreAPIRefundListener, CoreAPIReversalListener, CoreAPIUpdateListener, CoreAPIMessageListener, CoreAPIReportingListener {

	private EditText tvAmount;
	private ImageView connectedIcon;
	private RelativeLayout paymentLayout;
	private CanvasView signatureCanvas;
	private RelativeLayout canvasLinearLayout;
	private Animation fadeInInAnimation, slideInAnimation, slideOutAnimation;
	private String currentDevice;
	private InputMethodManager imm;
	private AlertDialog bluetoothSelection;
	private TextView tvSummary, tvStatus;
	private ViewType viewType = ViewType.SALE;
	private Button confirmButton, cancelButton, clearCustomFieldsBTN, customFieldsOkBTN;
	private View topView;
	private String nfcEnabledSupported;
	private boolean inTransaction = false;
	private CardDetailsFragment cardDetailsFragment;
	private ReceiptFragment receiptFragment;
	private CoreSale coreSale;
	private CheckBox debitCheckbox;
	private RelativeLayout customFieldLayout;
	private CheckBox addCustomFieldsCBX, bypassSurchargeCBX;
	private EditText customName1ET, customName2ET, customName3ET, customName4ET,
			customValue1ET, customValue2ET, customValue3ET, customValue4ET;

	private CheckBox addTipsAndTaxCheckBox;
	private Button customTipButton, btTaxTipOk, btTaxTipClear;
	private Spinner selectTipSpinnner, selectTaxSpinnner;
	private RelativeLayout tipsTaxLayout;
	private HashMap<String, CoreTax> taxMap = new HashMap<>();
	private HashMap<String, CoreTip> tipMap = new HashMap<>();
	private boolean needToGetSettings = false;
	private InputAccountDialog paxDialog;
	private AlertDialog pinDialog;
	private StringBuilder pinChars;
	private EditText pinEditText;
	private HashMap<String, String> deviceInfo;
	private boolean expiryDatePrompt = false;

	private TextWatcher amountTextWatcher;

	public void getSettings() {
		needToGetSettings = true;
	}

	public interface OnFragmentLoadedListener {
		void onFragmentLoaded();
	}

	private OnFragmentLoadedListener mListener;

	public void onAttach(Context context) {
		super.onAttach(context);
		try {
			this.mListener = (OnFragmentLoadedListener) context;
		} catch (final ClassCastException e) {
			throw new ClassCastException(context.toString() + " must implement OnCompleteListener");
		}
	}

	@Override
	public void onAmountUpdated(double newValue) {
		calculateAllAmounts();
	}

	public enum ViewType {
		SALE,
		REFUND
	}

	public void setViewType(ViewType viewType) {
		this.viewType = viewType;
	}

	private void setUI() {
		setColor(R.color.colorPrimary);

		if (tvSummary != null) {
			String text = (viewType.toString()).contains("_") ? viewType.toString().toUpperCase().replace("_", " ") : viewType.toString().toUpperCase();
			tvSummary.setText(text);
			confirmButton.setText(text);
		}

		if (viewType != ViewType.SALE) {
			cancelButton.setVisibility(View.GONE);
		}
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
							 Bundle savedInstanceState) {
		// Inflate the layout for this fragment
		View view = inflater.inflate(R.layout.fragment_payment,
				container, false);
		AndroidTerminal.getInstance().registerCoreAPIDeviceListener(this);
		AndroidTerminal.getInstance().registerCoreAPISaleListener(this);
		AndroidTerminal.getInstance().registerCoreAPIRefundListener(this);
		AndroidTerminal.getInstance().registerCoreAPIReversalListener(this);
		AndroidTerminal.getInstance().registerCoreAPIUpdateListener(this);
		AndroidTerminal.getInstance().registerCoreAPIMessageListener(this);
		AndroidTerminal.getInstance().registerCoreAPIReportingListener(this);

		slideInAnimation = AnimationUtils.loadAnimation(getActivity(),
				R.anim.slidein);
		slideOutAnimation = AnimationUtils.loadAnimation(getActivity(),
				R.anim.slideout);

		customName1ET = (EditText) view.findViewById(R.id.et_custom_name1);
		customName2ET = (EditText) view.findViewById(R.id.et_custom_name2);
		customName3ET = (EditText) view.findViewById(R.id.et_custom_name3);
		customName4ET = (EditText) view.findViewById(R.id.et_custom_name4);
		customValue1ET = (EditText) view.findViewById(R.id.et_custom_value1);
		customValue2ET = (EditText) view.findViewById(R.id.et_custom_value2);
		customValue3ET = (EditText) view.findViewById(R.id.et_custom_value3);
		customValue4ET = (EditText) view.findViewById(R.id.et_custom_value4);
		clearCustomFieldsBTN = (Button) view.findViewById(R.id.btn_clear);
		clearCustomFieldsBTN.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				customName1ET.setText("");
				customName2ET.setText("");
				customName3ET.setText("");
				customName4ET.setText("");
				customValue1ET.setText("");
				customValue2ET.setText("");
				customValue3ET.setText("");
				customValue4ET.setText("");
			}
		});
		customFieldsOkBTN = (Button) view.findViewById(R.id.btn_ok);
		customFieldsOkBTN.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				boolean hasCustomFields = !TextUtils.isEmpty(customValue1ET.getText());
				addCustomFieldsCBX.setChecked(hasCustomFields);
				addCustomFieldsCBX.setText(hasCustomFields ? "Custom fields added" : "Add custom fields");
				customFieldLayout.setVisibility(View.GONE);
				customFieldLayout.startAnimation(slideOutAnimation);
			}
		});
		customFieldLayout = (RelativeLayout) view.findViewById(R.id.customFieldsLayout);
		addCustomFieldsCBX = view.findViewById(R.id.addCustomFieldsCheckBox);
		addCustomFieldsCBX.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
				customFieldLayout.setVisibility(View.VISIBLE);
				customFieldLayout.startAnimation(slideInAnimation);
			}
		});

		bypassSurchargeCBX = view.findViewById(R.id.bypassSurchargeCheckBox);
		bypassSurchargeCBX.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
				if (coreSale != null && AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.BBPOSDEVICE)) {
					coreSale.setBypassSurcharge(b);
				}
			}
		});

		tipsTaxLayout = (RelativeLayout) view.findViewById(R.id.tipsTaxLayout);
		selectTipSpinnner = (Spinner) view.findViewById(R.id.selectTipSpinnner);
		selectTaxSpinnner = (Spinner) view.findViewById(R.id.selectTaxSpinnner);
		customTipButton = (Button) view.findViewById(R.id.customTipButton);
		customTipButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
				builder.setTitle("Amount");
				final EditText input = new EditText(getActivity());
				input.setInputType(InputType.TYPE_CLASS_NUMBER);
				builder.setView(input);
				builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						customTipButton.setText(input.getText().toString());
					}
				});
				builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						dialog.cancel();
					}
				});

				builder.show();
			}
		});
		tvStatus = (TextView) view.findViewById(R.id.tvStatus);

		addTipsAndTaxCheckBox = view.findViewById(R.id.addTipsAndTaxCheckBox);
		addTipsAndTaxCheckBox.setVisibility(View.GONE);
		addTipsAndTaxCheckBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
				if (tvAmount.getText().length() == 0 && !AndroidTerminal.getInstance().isDelayedAuthEnabled()) {
					addTipsAndTaxCheckBox.setChecked(false);
					if (!tvStatus.getText().toString().equalsIgnoreCase("Sale Response")) {
						UiHelper.getInstance().showWarning(getActivity(), "Amount empty.", "Amount field cannot be empty.");
					}
					return;
				}
				tipsTaxLayout.setVisibility(View.VISIBLE);
				tipsTaxLayout.startAnimation(slideInAnimation);
			}
		});
		btTaxTipClear = (Button) view.findViewById(R.id.btTaxTipClear);
		btTaxTipClear.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				selectTaxSpinnner.setSelection(0);
				selectTipSpinnner.setSelection(0);
				customTipButton.setText("Custom tip");
			}
		});

		btTaxTipOk = (Button) view.findViewById(R.id.btTaxTipOk);
		btTaxTipOk.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				calculateAllAmounts();
				tipsTaxLayout.setVisibility(View.GONE);
				tipsTaxLayout.startAnimation(slideOutAnimation);
			}
		});

		tvSummary = (TextView) view.findViewById(R.id.tvSummary);
		confirmButton = (Button) view.findViewById(R.id.btConfirm);
		confirmButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (viewType.equals(ViewType.SALE)) {
					doSale();
				} else if (viewType.equals(ViewType.REFUND)) {
					doRefund();
				}
			}
		});
		cancelButton = (Button) view.findViewById(R.id.btCancel);
		cancelButton.setVisibility(View.GONE);
		cancelButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (inTransaction) {
					AndroidTerminal.getInstance().cancelTransaction();
				}
			}
		});
		debitCheckbox = (CheckBox) view.findViewById(R.id.debitCheckbox);
		topView = (View) view.findViewById(R.id.topView);
		tvAmount = (EditText) view.findViewById(R.id.tvAmountView);

		inTransaction = false;
		imm = (InputMethodManager)
				getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
		paymentLayout = (RelativeLayout) view.findViewById(R.id.paymentLayout);
		canvasLinearLayout = (RelativeLayout) view.findViewById(R.id.llCanvas);
		signatureCanvas = (CanvasView) view.findViewById(R.id.signature_canvas);
		fadeInInAnimation = AnimationUtils.loadAnimation(getActivity(),
				R.anim.fadein);
		connectedIcon = (ImageView) view.findViewById(R.id.ivConnected);
		connectedIcon.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) { // connected
					final CharSequence[] items = {"Reconnect", "Get Device Info"};
					AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
					builder.setTitle("Sale");
					builder.setItems(items, new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int item) {
							if (item == 0) { //Reconnect
								connectMenu();
							} else {
								AndroidTerminal.getInstance().getDeviceInfo();
							}
							imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
						}
					});
					AlertDialog alert = builder.create();
					alert.show();
				} else {
					connectMenu();
				}
			}
		});

		Button rejectSignatureButton = view.findViewById(R.id.btSigReject);
		if (rejectSignatureButton != null) {
			rejectSignatureButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View view) {
					AndroidTerminal.getInstance().cancelTransaction();
					dismissSignatureScreen();
					UiHelper.getInstance().showError(getActivity(), getString(R.string.transaction_declined), getString(R.string.transaction_not_authorized));
				}
			});
		}

		Button cancelSignatureButton = view.findViewById(R.id.btSigCancel);
		if (cancelSignatureButton != null) {
			cancelSignatureButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View view) {
					AndroidTerminal.getInstance().cancelTransaction();
					dismissSignatureScreen();
				}
			});
		}

		Button submitSignature = (Button) view.findViewById(R.id.btSignature);
		if (submitSignature != null) {
			submitSignature.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					if (signatureCanvas.checkSignature()) {

						DialogInterface.OnClickListener signaturePositiveAction = new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialogInterface, int which) {
								canvasLinearLayout.setVisibility(View.GONE);
								paymentLayout.setVisibility(View.VISIBLE);
								paymentLayout.startAnimation(fadeInInAnimation);
								UiHelper.getInstance().showDialog("PROCESSING", getActivity(), false);

								AndroidTerminal.getInstance().submitSignature(signatureCanvas.getSignature());
								if (signatureCanvas != null) {
									signatureCanvas.clearCanvas();
								}
							}
						};

						DialogInterface.OnClickListener signatureNegativeAction = new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialogInterface, int which) {

							}
						};


						UiHelper.getInstance().showSignatureConfirmation(getActivity(), signaturePositiveAction, signatureNegativeAction);
					} else {
						UiHelper.getInstance().showWarning(getActivity(), "Signature empty.", "Signature cannot be empty.");
					}
				}
			});
		}
		if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) { // connected
			connectedIcon.setImageDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.connected));
		}
		setUI();

		if (AndroidTerminal.getInstance().getSettings() != null) {
			tvAmount.setHint(AndroidTerminal.getInstance().getSettings().getCurrency().getSymbol() + (new BigDecimal(0)).setScale(AndroidTerminal.getInstance().getSettings().getCurrency().getExponent(), RoundingMode.HALF_EVEN).toString());
			changeTvAmountListener(Util.addCurrencyMask(AndroidTerminal.getInstance().getSettings().getCurrency(), this, tvAmount));
			configureTipAndTaxUi(AndroidTerminal.getInstance().getSettings());
		}

		if (mListener != null && needToGetSettings) {
			mListener.onFragmentLoaded();
			needToGetSettings = false;
		}
		return view;
	}

	public boolean onKeyUp(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_ENTER && event.getAction() == KeyEvent.ACTION_UP &&
				deviceInfo != null && deviceInfo.containsKey("modelName") && "A80".equals(deviceInfo.get("modelName"))) {
			if (!expiryDatePrompt) {
				saleWithPax();
			}
			expiryDatePrompt = false;
		}
		return true;
	}

	private void dismissSignatureScreen() {
		canvasLinearLayout.setVisibility(View.GONE);
		paymentLayout.setVisibility(View.VISIBLE);
		paymentLayout.startAnimation(fadeInInAnimation);
	}

	private void connectMenu() {
		LinkedHashMap<String, String> supportedDevices = AndroidTerminal.getInstance().listSupportedDevices();
		AlertDialog.Builder builder = new AlertDialog.Builder(
				getActivity());
		builder.setTitle("Devices");
		final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<>(
				getActivity(),
				android.R.layout.select_dialog_item);
		arrayAdapter.clear();
		for (String device : supportedDevices.keySet()) {
			arrayAdapter.add(device);
		}
		builder.setNegativeButton("cancel",
				new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						dialog.dismiss();
					}
				});

		builder.setAdapter(arrayAdapter,
				new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						currentDevice = arrayAdapter.getItem(which);
						if (currentDevice.equals("Bluepad")) {
							Toast.makeText(getActivity(), "Initializing Pinpad.", Toast.LENGTH_SHORT).show();
							AndroidTerminal.getInstance().initDevice(DeviceEnum.BLUEPAD, DeviceConnectionType.BLUETOOTH, new HashMap<String, Object>());
						} else if (currentDevice.equals("BBPosDevice")) {
							//AndroidTerminal.getInstance().initDevice(DeviceEnum.BBPOSDEVICE, DeviceConnectionType.AUDIO, "2F:1F:00:80:2B:50");
							final CharSequence[] items = {"BLUETOOTH", "AUDIO"};
							AlertDialog.Builder builder1 = new AlertDialog.Builder(getActivity());
							builder1.setTitle("Sale");
							builder1.setItems(items, new DialogInterface.OnClickListener() {
								public void onClick(DialogInterface dialog, int item) {
									if (item == 0) {
										Toast.makeText(getActivity(), "Initializing BBPosDevice.", Toast.LENGTH_SHORT).show();

										HashMap<String, Object> data = new HashMap<>();
										//data.put("emvType", CoreEmvType.QUICK_CHIP);
										//data.put("forceReconnect", false);
										//data.put("enableTips", true);
										//data.put("tipPreDefinedAmount", 12);
										//data.put("enableSurcharges", true);
										AndroidTerminal.getInstance().initDevice(DeviceEnum.BBPOSDEVICE, DeviceConnectionType.BLUETOOTH,  data);

										final CharSequence[] items = {"Scanning..."};
										AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
										builder.setTitle("Bluetooth devices");
										builder.setItems(items, new DialogInterface.OnClickListener() {
											public void onClick(DialogInterface dialog, int item) {
											}
										});
										builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
											@Override
											public void onClick(DialogInterface dialogInterface, int i) {
											}
										});
										bluetoothSelection = builder.create();
										bluetoothSelection.show();
									} else if (item == 1) {
										Toast.makeText(getActivity(), "Initializing BBPosDevice.", Toast.LENGTH_SHORT).show();
										AndroidTerminal.getInstance().initDevice(DeviceEnum.BBPOSDEVICE, DeviceConnectionType.AUDIO, new HashMap<String, Object>());
									}
								}
							});
							AlertDialog alert = builder1.create();
							alert.show();
						} else if (currentDevice.equals("Idtech")) {
							Toast.makeText(getActivity(), "Initializing IdTech.", Toast.LENGTH_SHORT).show();

							HashMap<String, Object> data = new HashMap<>();
							//data.put("emvType", CoreEmvType.QUICK_CHIP);
							//data.put("forceReconnect", false);
							//data.put("selfCheckTime", "0000");
							//data.put("inputMethod", CoreTransactionInputMethod.SWIPE_OR_INSERT);
							//data.put("enableDeviceLogs", true);
							//data.put("vasSmartTapConfig", new CoreVasSmartTapConfig.Builder()
							//		.setGoogleSmartTapCollectorId(87133300)
							//		.setGoogleSmartTapPrivateKeyVersion(10)
							//		.setGoogleSmartTapPrivateKey("F5368708933920553B7B9FFB16AEED9C77D5BFD9662AF149A6B9F965B73F0CCA")
							//		.addAppleVasMerchant("pass.com.pronto.id-tech.demo", "www.idtechproducts.com")
							//		.setAppleVasPrivateKey("F5368708933920553B7B9FFB16AEED9C77D5BFD9662AF149A6B9F965B73F0CCA")
							//		.build());
							AndroidTerminal.getInstance().initDevice(DeviceEnum.IDTECH, DeviceConnectionType.USB, null, data);
						} else if (currentDevice.equals("Ingenico")) {
							HashMap<String, Object> data = new HashMap<>();
							//data.put("emvType", CoreEmvType.QUICK_CHIP);
							//data.put("enableTips", true);
							//data.put("tipPreDefinedAmount", 12);
							//data.put("customTips", new CoreCustomTips(createTips(TipType.PERCENTAGE), createTips(TipType.FIXED_AMOUNT)));
							//data.put("forceReconnect", false);
							data.put("inputMethod", CoreTransactionInputMethod.SWIPE_OR_INSERT);
							AndroidTerminal.getInstance().initDevice(DeviceEnum.INGENICO, DeviceConnectionType.USB_CDC, null, data);
						} else if (currentDevice.equals("Pax")) {
							HashMap<String, Object> data = new HashMap<>();
							//data.put("emvType", CoreEmvType.QUICK_CHIP);
							data.put("performCardExpirationCheckInOfflineMode", true);
							//data.put("externalPinpad", true);
							AndroidTerminal.getInstance().initDevice(DeviceEnum.PAX, DeviceConnectionType.AIDL, null, data);
						}
						imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
					}
				});
		builder.show();
	}

	private List<CoreTip> createTips(TipType tipType) {
		List<CoreTip> tips = new ArrayList<>();
		for (int i = 0; i < 3; i++) {
			CoreTip tip = new CoreTip();
			tip.setTipType(tipType);
			if (tipType == TipType.PERCENTAGE) {
				tip.setPercentage(new BigDecimal(3.0 * (i + 1)));
			} else {
				tip.setAmount(new BigDecimal(2.0 * (i + 1)));
			}
			tips.add(tip);
		}
		return tips;
	}

	private void doSale() {
		if (tvAmount.getText().length() == 0 && !AndroidTerminal.getInstance().isDelayedAuthEnabled()) {
			UiHelper.getInstance().showWarning(getActivity(), "Amount empty.", "Amount field cannot be empty.");
			return;
		}
		if (signatureCanvas != null) {
			signatureCanvas.clearCanvas();
		}
		if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.NODEVICE)) {
			if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.IDTECH)) {
				UiHelper.getInstance().showDialog("STARTING TRANSACTION", getActivity(), true);
				if (AndroidTerminal.getInstance().isDelayedAuthEnabled()) {
					coreSale = new CoreSale(); // delayed auth, amount will be set in onRequestAmount
				} else {
					coreSale = new CoreSale(CoreUtil.parseStringToBigDecimal(tvAmount.getText().toString()));
				}
				addCustomFieldsToCoreSale(coreSale);
				coreSale.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE_OR_INSERT_OR_TAP);
				inTransaction = true;
				AndroidTerminal.getInstance().processSale(coreSale);
				imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
			} else if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)) {
				saleWithPax();
			} else {
				List<String> listItems = new ArrayList<>();
				listItems.add("Insert/Swipe");
				listItems.add("Swipe");
				listItems.add("TAP");
				listItems.add("Quick Chip");
				listItems.add("Manual Entry No Device");
				listItems.add("Manual entry");
				listItems.add("Swipe/Insert/Tap");
				final CharSequence[] items = listItems.toArray(new CharSequence[listItems.size()]);

				AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
				builder.setTitle("Sale");
				builder.setItems(items, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int item) {
						if (item == 4) {
							boolean isKeyed = AndroidTerminal.getInstance().getSettings() != null ? AndroidTerminal.getInstance().getSettings().getFeatures().get("allowKeyed") : false;
							if (!isKeyed) {
								UiHelper.getInstance().showWarning(getActivity(), "Keyed not enabled", "Keyed functionality is not enabled in the selfcare system.");
								return;
							}
							coreSale = new CoreSale(BigDecimal.valueOf(Double.parseDouble(tvAmount.getText().toString().replaceAll("[^0-9.]", ""))));
							if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.BBPOSDEVICE) && bypassSurchargeCBX.getVisibility() == View.VISIBLE) {
								if (coreSale.getBypassSurcharge() == null) {
									coreSale.setBypassSurcharge(bypassSurchargeCBX.isChecked());
								}
							}
							cardDetailsFragment = CardDetailsFragment.getInstance(CardDetailsFragment.ViewType.SALE, coreSale, null, null, null, null, null);
							getActivity().getSupportFragmentManager()
									.beginTransaction()
									.add(R.id.content_frame, cardDetailsFragment)
									.addToBackStack(null)
									.commit();
							return;
						}
						if (AndroidTerminal.getInstance().isDelayedAuthEnabled()) {
							coreSale = new CoreSale(); // delayed auth, amount will be set in onRequestAmount
						} else {
							coreSale = new CoreSale(BigDecimal.valueOf(Double.parseDouble(tvAmount.getText().toString().replaceAll("[^0-9.]", ""))));
							if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.BBPOSDEVICE) && bypassSurchargeCBX.getVisibility() == View.VISIBLE) {
								if (coreSale.getBypassSurcharge() == null) {
									coreSale.setBypassSurcharge(bypassSurchargeCBX.isChecked());
								}
							}
						}
						addCustomFieldsToCoreSale(coreSale);
						UiHelper.getInstance().showDialog("STARTING TRANSACTION", getActivity(), true);
						switch (item) {
							case 0:
								coreSale.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE_OR_INSERT);
								break;
							case 1:
								coreSale.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE);
								break;
							case 2:
								coreSale.setTransactionInputMethod(CoreTransactionInputMethod.TAP);
								break;
							case 3:
								coreSale.setEmvType(CoreEmvType.QUICK_CHIP);
								break;
							case 5:
								coreSale.setTransactionInputMethod(CoreTransactionInputMethod.MANUAL_PAN_ENTRY);
								break;
							case 6:
								coreSale.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE_OR_INSERT_OR_TAP);
								break;
						}

						if (debitCheckbox.isChecked()) {
							coreSale.setDebitAccountType(DebitAccountType.CHECKING);
						}

						addTipOrTaxToSale(coreSale);
						coreSale.setSignatureCollection(SignatureCollection.AUTOMATIC);
						inTransaction = true;
						AndroidTerminal.getInstance().processSale(coreSale);
						imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
					}
				});
				AlertDialog alert = builder.create();
				alert.show();
			}
		} else {
			final CharSequence[] items = {"Keyed", "Android Pay"};
			AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
			builder.setTitle("Sale");
			builder.setItems(items, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int item) {
					if (item == 1) {
						AndroidPay.getInstance(getActivity()).setUp(tvAmount.getText().toString().replaceAll("[^0-9.]", ""));
					} else {
						boolean isKeyed = AndroidTerminal.getInstance().getSettings() != null ? AndroidTerminal.getInstance().getSettings().getFeatures().get("allowKeyed") : false;
						if (!isKeyed) {
							UiHelper.getInstance().showWarning(getActivity(), "Keyed not enabled", "Keyed functionality is not enabled in the selfcare system.");
							return;
						}
						coreSale = new CoreSale(BigDecimal.valueOf(Double.parseDouble(tvAmount.getText().toString().replaceAll("[^0-9.]", ""))));
						cardDetailsFragment = CardDetailsFragment.getInstance(CardDetailsFragment.ViewType.SALE, coreSale, null, null, null, null, null);
						getActivity().getSupportFragmentManager()
								.beginTransaction()
								.add(R.id.content_frame, cardDetailsFragment)
								.addToBackStack(null)
								.commit();
					}
					imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
				}
			});
			AlertDialog alert = builder.create();
			alert.show();
		}
	}

	private void saleWithPax() {
		if (tvAmount.getText().length() == 0 && !AndroidTerminal.getInstance().isDelayedAuthEnabled()) {
			UiHelper.getInstance().showWarning(getActivity(), "Amount empty.", "Amount field cannot be empty.");
			return;
		}
		final CharSequence[] items = {"EMV", "EBT"};
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("Sale");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					final CharSequence[] items = {"Standard", "Quick-Chip"};
					AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
					builder.setTitle("Sale");
					builder.setItems(items, new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int item) {
							coreSale = new CoreSale(CoreUtil.parseStringToBigDecimal(tvAmount.getText().toString()));
							inTransaction = true;
							displayPaxPopup("Swipe/Insert/Tap or Enter card number");
							coreSale.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE_OR_INSERT_OR_TAP_OR_KEYED);
							if (item == 1) {
								coreSale.setEmvType(CoreEmvType.QUICK_CHIP);
							} else {
								coreSale.setEmvType(CoreEmvType.STANDARD);
							}
							AndroidTerminal.getInstance().processSale(coreSale);
							imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
						}
					});
					AlertDialog alert = builder.create();
					alert.show();
				} else if (item == 1) {
					final CharSequence[] items = {"EBT Food Stamp", "EBT Cash Benefit", "EBT Cash Back", "EBT Cash Withdrawal", "EBT Food Voucher"
							, "EBT Balance Inquiry", "Unreferenced Refund", "Unreferenced Refund voucher", "Reports"};
					AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
					builder.setTitle("Sale");
					builder.setItems(items, new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int item) {
							coreSale = new CoreSaleEbt(CoreUtil.parseStringToBigDecimal(tvAmount.getText().toString()));
							displayPaxPopup("Swipe Card or Enter card number");
							paxDialog.show();
							inTransaction = true;
							imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
							((CoreSaleEbt) coreSale).setOperator("android-operator");
							((CoreSaleEbt) coreSale).setDeviceId("12345678");
							if (item == 0) {
								((CoreSaleEbt) coreSale).setCardAccount(CardAccount.FOOD_STAMP);
								coreSale.setSaleRequestType(SaleRequestType.PURCHASE);
							} else if (item == 1) {
								((CoreSaleEbt) coreSale).setCardAccount(CardAccount.CASH);
								coreSale.setSaleRequestType(SaleRequestType.PURCHASE);
							} else if (item == 2) {
								((CoreSaleEbt) coreSale).setCardAccount(CardAccount.CASH);
								((CoreSaleEbt) coreSale).setCashBackAmount(BigDecimal.ONE);
								coreSale.setSaleRequestType(SaleRequestType.PURCHASE);
							} else if (item == 3) {
								((CoreSaleEbt) coreSale).setCardAccount(CardAccount.CASH);
								coreSale.setSaleRequestType(SaleRequestType.CASH_WITHDRAWAL);
							} else if (item == 4) { // EBT Food Voucher
								paxDialog.dismiss();
								CoreVoucher voucher = new CoreVoucher();
								voucher.setCardAccount(CardAccount.FOOD_STAMP);
								voucher.setVoucherApprovalCode("123456");
								voucher.setVoucherNumber("123456789012345");
								voucher.setCardNumber("6007602801003837968");
								((CoreSaleEbt) coreSale).setVoucher(voucher);
								coreSale.setSaleRequestType(SaleRequestType.PURCHASE);
							} else if (item == 5) {
								CoreBalanceInquiryEbt ebt = new CoreBalanceInquiryEbt();
								ebt.setCardAccount(CardAccount.FOOD_STAMP);
								ebt.setOperator("android-operator");
								ebt.setDeviceId("12345678");
								AndroidTerminal.getInstance().processBalanceInquiry(ebt);
								return;
							} else if (item == 6) {
								CoreUnreferencedRefundEbt ebt = new CoreUnreferencedRefundEbt(CoreUtil.parseStringToBigDecimal(tvAmount.getText().toString()));
								ebt.setCardAccount(CardAccount.FOOD_STAMP);
								ebt.setReason("reason");
								ebt.setOperator("android-operator");
								ebt.setDeviceId("12345678");
								AndroidTerminal.getInstance().processUnreferencedRefund(ebt);
								return;
							} else if (item == 7) { // voucher refund
								paxDialog.dismiss();
								CoreUnreferencedRefundEbt ebt = new CoreUnreferencedRefundEbt(CoreUtil.parseStringToBigDecimal(tvAmount.getText().toString()));
								ebt.setOperator("android-operator");
								ebt.setDeviceId("12345678");
								CoreVoucher voucher = new CoreVoucher();
								voucher.setCardAccount(CardAccount.FOOD_STAMP);
								voucher.setCardNumber("6007602801003837968");
								voucher.setVoucherApprovalCode("123456");
								voucher.setVoucherNumber("123456789012345");
								ebt.setVoucher(voucher);
								ebt.setReason("reason");
								AndroidTerminal.getInstance().processUnreferencedRefund(ebt);
								return;
							} else if (item == 8) { // reports
								paxDialog.dismiss();
								AndroidTerminal.getInstance().getTransactionReports("25-11-2019", "OPERATOR", "android-operator");
								return;
							}
							coreSale.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE_OR_KEYED);
							AndroidTerminal.getInstance().processSale(coreSale);
						}
					});
					AlertDialog alert = builder.create();
					alert.show();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

	}

	private void addCustomFieldsToCoreSale(CoreSale coreSale) {
		CoreCustomField field;
		List<CoreCustomField> customFields = null;
		if (!customName1ET.getText().toString().trim().isEmpty()) {
			if (customValue1ET.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 1", Toast.LENGTH_SHORT).show();
				return;
			}
			if (customFields == null) {
				customFields = new ArrayList<>();
			}
			field = new CoreCustomField(customName1ET.getText().toString(), customValue1ET.getText().toString());
			customFields.add(field);
		}
		if (!customName2ET.getText().toString().trim().isEmpty()) {
			if (customValue2ET.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 2", Toast.LENGTH_SHORT).show();
				return;
			}
			if (customFields == null) {
				customFields = new ArrayList<>();
			}
			field = new CoreCustomField(customName2ET.getText().toString(), customValue2ET.getText().toString());
			customFields.add(field);
		}
		if (!customName3ET.getText().toString().trim().isEmpty()) {
			if (customValue3ET.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 3", Toast.LENGTH_SHORT).show();
				return;
			}
			if (customFields == null) {
				customFields = new ArrayList<>();
			}
			field = new CoreCustomField(customName3ET.getText().toString(), customValue3ET.getText().toString());
			customFields.add(field);
		}
		if (!customName4ET.getText().toString().trim().isEmpty()) {
			if (customValue4ET.getText().toString().trim().isEmpty()) {
				Toast.makeText(getActivity(), "Please enter value for custom field 4", Toast.LENGTH_SHORT).show();
				return;
			}
			if (customFields == null) {
				customFields = new ArrayList<>();
			}
			field = new CoreCustomField(customName4ET.getText().toString(), customValue4ET.getText().toString());
			customFields.add(field);
		}
		if (customFields != null) {
			coreSale.setCustomFields(customFields);
		}
	}

	private void addTipOrTaxToSale(CoreSale coreSale) {
		if (addTipsAndTaxCheckBox.getVisibility() == View.VISIBLE) {
			if (selectTaxSpinnner.getSelectedItemPosition() != 0 && selectTaxSpinnner.getSelectedItem() != null) {
				String percentage = selectTaxSpinnner.getSelectedItem().toString();
				CoreTax tax = taxMap.get(percentage);
				coreSale.addTax(tax);
			}
			if (selectTipSpinnner.getSelectedItemPosition() != 0 && selectTipSpinnner.getSelectedItem() != null) {
				String percentage = selectTipSpinnner.getSelectedItem().toString();
				CoreTip tip = tipMap.get(percentage);
				coreSale.setTip(tip);
			}
			if (!customTipButton.getText().equals("Custom tip")) {
				if (coreSale.getTip() == null) {
					CoreTip tip = new CoreTip();
					tip.setTipType(TipType.FIXED_AMOUNT);
					tip.setAmount(BigDecimal.valueOf(Double.parseDouble(customTipButton.getText().toString())));
					coreSale.setTip(tip);
				}
			}
		}
	}

	private void calculateAllAmounts() {
		if (addTipsAndTaxCheckBox.getVisibility() == View.VISIBLE) {
			boolean taxOrTipAdded = selectTaxSpinnner.getSelectedItemPosition() > 0 || selectTipSpinnner.getSelectedItemPosition() > 0 || !customTipButton.getText().equals("Custom tip");
			addTipsAndTaxCheckBox.setChecked(taxOrTipAdded);
			String message = "Tips or taxes added";
			if (taxOrTipAdded) {
				if (tvAmount.getText().toString().equals("")) {
					addTipsAndTaxCheckBox.setChecked(false);
					addTipsAndTaxCheckBox.setText("Add tip and tax");
					return;
				}
				CoreSale coreSale = new CoreSale(BigDecimal.valueOf(Double.parseDouble(tvAmount.getText().toString().replaceAll("[^0-9.]", ""))));
				addTipOrTaxToSale(coreSale);
				message = message + ". Total Amount: " + CoreUtil.addUpAllAmounts(coreSale, AndroidTerminal.getInstance().getCurrency());
			}
			addTipsAndTaxCheckBox.setText(taxOrTipAdded ? message : "Add tip and tax");
		}
	}

	private void doRefund() {
		if (tvAmount.getText().length() == 0) {
			UiHelper.getInstance().showWarning(getActivity(), "Amount empty.", "Amount field cannot be empty.");
			return;
		}
		CoreUnreferencedRefund coreUnreferenced = new CoreUnreferencedRefund(BigDecimal.valueOf(Double.parseDouble(tvAmount.getText().toString().replaceAll("[^0-9.]", ""))));
		if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.INGENICO) || AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)) {
			if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)){
				coreUnreferenced.setTransactionInputMethod(CoreTransactionInputMethod.SWIPE_OR_INSERT_OR_TAP_OR_KEYED);
			}
			if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)){
				displayPaxPopup("Swipe/Insert/Tap or Enter card number");
			} else {
				UiHelper.getInstance().showDialog("STARTING TRANSACTION", getActivity(), true);
			}
			inTransaction = true;
			AndroidTerminal.getInstance().processUnreferencedRefund(coreUnreferenced);
		} else {
			cardDetailsFragment = CardDetailsFragment.getInstance(CardDetailsFragment.ViewType.REFUND, null, coreUnreferenced, null, null, null, null);
			getActivity().getSupportFragmentManager()
					.beginTransaction()
					.add(R.id.content_frame, cardDetailsFragment)
					.addToBackStack(null)
					.commit();
			imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
		}
	}

	private void setColor(int id) {
		if (topView != null) {
			topView.setBackgroundColor(getResources().getColor(id));
		}
		if (confirmButton != null) {
			confirmButton.setBackgroundColor(getResources().getColor(id));
		}
		if (tvAmount != null) {
			tvAmount.setTextColor(getResources().getColor(id));
		}
	}

	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		AndroidPay.getInstance(getActivity()).onActivityResult(requestCode, resultCode, data);
		super.onActivityResult(requestCode, resultCode, data);
	}

	private void changeTvAmountListener(TextWatcher watcher) {
		if(amountTextWatcher != null) {
			tvAmount.removeTextChangedListener(amountTextWatcher);
		}
		amountTextWatcher = watcher;
		tvAmount.addTextChangedListener(amountTextWatcher);
	}

	public void onSettingsRetrieved(CoreSettings settings) {
		tvStatus.setText("Settings retrieved");
		tvAmount.setHint(settings.getCurrency().getSymbol() + (new BigDecimal(0)).setScale(settings.getCurrency().getExponent(), RoundingMode.HALF_EVEN).toString());
		changeTvAmountListener(Util.addCurrencyMask(settings.getCurrency(), this, tvAmount));
		configureTipAndTaxUi(settings);
		bypassSurchargeCBX.setVisibility(settings.getFeatures().get("allowSurcharges") && settings.getFeatures().get("allowSurchargesForMobile") ? View.VISIBLE : View.GONE);
	}

	private void configureTipAndTaxUi(CoreSettings settings) {
		if (settings.getFeatures().get("allowTip") || settings.getFeatures().get("allowTax")) {
			ArrayList<CoreSetting> taxList = settings.getTaxList();
			if (taxList.size() != 0) {
				taxMap = new HashMap<>();
				ArrayList<String> spinnerArray = new ArrayList<String>();
				addTipsAndTaxCheckBox.setVisibility(View.VISIBLE);
				spinnerArray.add("Select tax");
				for (CoreSetting tax : taxList) {
					String percentage = String.valueOf(((CoreTax) tax).getPercentage());
					taxMap.put(percentage, (CoreTax) tax);
					spinnerArray.add(percentage);
				}
				ArrayAdapter<String> spinnerArrayAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item, spinnerArray);
				selectTaxSpinnner.setAdapter(spinnerArrayAdapter);
			}

			ArrayList<CoreSetting> tipList = settings.getTipList();
			if (tipList.size() != 0) {
				tipMap = new HashMap<>();
				ArrayList<String> spinnerArray1 = new ArrayList<String>();
				addTipsAndTaxCheckBox.setVisibility(View.VISIBLE);
				spinnerArray1.add("Select tip");
				for (CoreSetting tip : tipList) {
					if (((CoreTip) tip).getTipType().equals(TipType.PERCENTAGE)) {
						String percentage = String.valueOf(((CoreTip) tip).getPercentage());
						tipMap.put(percentage, (CoreTip) tip);
						spinnerArray1.add(percentage);
					}
				}
				ArrayAdapter<String> spinnerArrayAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item, spinnerArray1);
				selectTipSpinnner.setAdapter(spinnerArrayAdapter);
			}
		} else {
			addTipsAndTaxCheckBox.setVisibility(View.GONE);
		}
	}

	@Override
	public void onMessage(CoreMessage message) {
		checkPaxDialog(message);
		//System.out.println(String.format("CoreMessage: %s", message));
		String displayMessage = message.name().replace("_", " ");
		tvStatus.setText(displayMessage);

		if ((message == CoreMessage.CARD_REQUIRES_CHIP_READ
			|| message == CoreMessage.SELF_CHECK_REBOOT) && !AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)) {
			inTransaction = false;
			cancelButton.setVisibility(View.GONE);
			UiHelper.getInstance().dismissProgress();
		} else if (AndroidTerminal.getInstance().isPollingEnabled()) {
			if (UiHelper.getInstance().isProgressShowing()) {
				UiHelper.getInstance().updateMessage(displayMessage);
			}
		} else if (AndroidTerminal.getInstance().isPollingEnabled() && message != CoreMessage.REMOVE_CARD) {
			if (UiHelper.getInstance().isProgressShowing()) {
				UiHelper.getInstance().updateMessage(displayMessage);
			} else {
				UiHelper.getInstance().showDialog(displayMessage, getActivity(), false);
			}
		} else if (message == CoreMessage.RECONNECTING) {
			if (bluetoothSelection != null && bluetoothSelection.isShowing()) {
				bluetoothSelection.dismiss();
			}
		} else {
			if (inTransaction && UiHelper.getInstance().isProgressShowing()) {
				UiHelper.getInstance().updateMessage(displayMessage);
			} else {
				Toast.makeText(getActivity(), displayMessage, Toast.LENGTH_SHORT).show();
			}
		}
	}

	private void checkPaxDialog(CoreMessage message) {
		if (paxDialog != null && paxDialog.isShowing()
				&& message != CoreMessage.SWIPE_OR_INSERT_OR_TAP_OR_KEYED_CARD
				&& message != CoreMessage.SWIPE_OR_KEYED_CARD
				&& message != CoreMessage.SWIPE_OR_INSERT_OR_KEYED_CARD
				&& message != CoreMessage.INSERT_OR_SWIPE_OR_TAP_CARD
				&& message != CoreMessage.KEYED_CARD
				&& message != CoreMessage.TRANSACTION_CANCELLED_BY_USER) {
			paxDialog.dismiss();
			UiHelper.getInstance().showDialog("PROCESSING", getActivity(), true);
		}
		if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX) &&
				message == CoreMessage.CARD_REQUIRES_CHIP_READ){
			displayPaxPopup("Swipe/Insert/Tap or Enter card number");
		}
	}

	private void displayPaxPopup(String text){
		paxDialog = new InputAccountDialog(getActivity(), tvAmount.getText().toString(), text, new InputAccountDialog.DialogCallback() {
			@Override
			public void onConfirmCardNum(String cardNum) {
				paxDialog.dismiss();
				AndroidTerminal.getInstance().submitCardNumber(cardNum);
			}

			@Override
			public void onCancelButtonPressed() {
				paxDialog.dismiss();
			}
		});
		paxDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialog) {
				paxDialog.dismiss();
				AndroidTerminal.getInstance().cancelTransaction();
			}
		});
		paxDialog.show();
	}

	@Override
	public void onSaleResponse(CoreSaleResponse response) {
		tvStatus.setText("Sale Response");
		if (addTipsAndTaxCheckBox.getVisibility() == View.VISIBLE) {
			selectTaxSpinnner.setSelection(0);
			selectTipSpinnner.setSelection(0);
			customTipButton.setText("Custom tip");
		}
		tvAmount.setText("");
		UiHelper.getInstance().dismissProgress();
		if (cardDetailsFragment != null && cardDetailsFragment.isAdded()) {
			getActivity().getSupportFragmentManager().beginTransaction().remove(cardDetailsFragment).commit();
		}
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		receiptFragment = ReceiptFragment.getInstance(response, coreSale, response.getInputMethod());
		getActivity().getSupportFragmentManager()
				.beginTransaction()
				.add(R.id.content_frame, receiptFragment)
				.addToBackStack(null)
				.commit();
	}

	@Override
	public void onRefundResponse(CoreRefundResponse response) {
		if (paxDialog != null) {
			paxDialog.dismiss();
		}
		tvAmount.setText("");
		tvStatus.setText("Refund Response");
		UiHelper.getInstance().dismissProgress();
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		if (cardDetailsFragment != null && cardDetailsFragment.isAdded()) {
			getActivity().getSupportFragmentManager().beginTransaction().remove(cardDetailsFragment).commit();
		}

		receiptFragment = ReceiptFragment.getInstance(response, "", "UNREFERENCED", "");
		getActivity().getSupportFragmentManager()
				.beginTransaction()
				.add(R.id.content_frame, receiptFragment)
				.addToBackStack(null)
				.commit();
	}

	@Override
	public void onSignatureRequired(CoreSignature signature) {
		tvStatus.setText("Signature required");
		UiHelper.getInstance().dismissProgress();
		signatureCanvas.setSignature(signature);
		paymentLayout.setVisibility(View.GONE);
		canvasLinearLayout.setVisibility(View.VISIBLE);
		canvasLinearLayout.startAnimation(fadeInInAnimation);
	}

	@Override
	public void onError(CoreError error, String message) {
		if (paxDialog != null) {
			paxDialog.dismiss();
		}
		if (tvStatus == null) {
			return;
		}
		if (addTipsAndTaxCheckBox.getVisibility() == View.VISIBLE) {
			selectTaxSpinnner.setSelection(0);
			selectTipSpinnner.setSelection(0);
			customTipButton.setText("Custom tip");
		}
		tvStatus.setText("Error " + error.name());
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		UiHelper.getInstance().showError(getActivity(), error.name().replace("_", ""), message);
	}

	@Override
	public void onDeviceError(CoreDeviceError error, String message) {
		if (error == CoreDeviceError.TRANSACTION_ABORTED && paxDialog != null){
			return;
		}
		if (error == CoreDeviceError.CARD_NUMBER_INCORRECT_LENGTH_PLEASE_RETRY){
			tvStatus.setText("Device Error " + error.name());
			Toast.makeText(getActivity(), "Card number incorrect please retry", Toast.LENGTH_SHORT).show();
			displayPaxPopup("Swipe/Insert/Tap or Enter card number");
			return;
		}
		if (paxDialog != null) {
			paxDialog.dismiss();
		}
		tvStatus.setText("Device Error " + error.name());
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		if (error == CoreDeviceError.TIMEOUT && AndroidTerminal.getInstance().isPollingEnabled()) {
			return;
		}
		UiHelper.getInstance().showError(getActivity(), error.name().replace("_", ""), message);
	}

	@Override
	public void onDeviceConnected(DeviceEnum deviceEnum, HashMap<String, String> deviceInfo) {
		nfcEnabledSupported = deviceInfo.get("isSupportedNfc");
		this.deviceInfo = deviceInfo;
		tvStatus.setText("Device connected " + deviceEnum.name());
		connectedIcon.setImageDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.connected));
		UiHelper.getInstance().showSuccess(getActivity(), "Device connected", deviceEnum.name() + " connected");
		//AndroidTerminal.getInstance().enableDelayedAuth(BigDecimal.valueOf(10));
		//if (deviceEnum.equals(DeviceEnum.IDTECH)) {
//			AndroidTerminal.getInstance().enablePolling(BigDecimal.valueOf(1), 60);
		//}
	}

	@Override
	public void onDeviceDisconnected(DeviceEnum deviceEnum) {
		tvStatus.setText("Device disconnected " + deviceEnum.name());
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		connectedIcon.setImageDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.disconnected));
		Toast.makeText(getActivity(), "Device disconnected " + deviceEnum.name() + " disconnected", Toast.LENGTH_SHORT).show();
		if (UiHelper.getInstance().isProgressShowing()) {
			UiHelper.getInstance().dismissProgress();
		}

	}

	@Override
	public void onSelectApplication(ArrayList<String> applications) {
		tvStatus.setText("Select application ");
		final CharSequence[] items = applications.toArray(new CharSequence[applications.size()]);
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("Applications");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				AndroidTerminal.getInstance().submitApplication(item);
				imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}

	@Override
	public void onSelectBTDevice(ArrayList<Object> devices) {
		tvStatus.setText("Select bluetooth device ");
		if (bluetoothSelection != null && bluetoothSelection.isShowing()) {
			bluetoothSelection.hide();
		}
		ArrayList<Object> deviceNames = new ArrayList<>();
		for (Object device : devices) {
			BluetoothDevice device1 = (BluetoothDevice) device;
			deviceNames.add(device1.getName());
		}
		CharSequence items[] = deviceNames.toArray(new CharSequence[devices.size()]);
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("Bluetooth devices");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				AndroidTerminal.getInstance().selectBTDevice(item);
				imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
			}
		});
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialogInterface, int i) {
				AndroidTerminal.getInstance().selectBTDevice(-1);
			}
		});
		bluetoothSelection = builder.create();
		bluetoothSelection.show();
	}

	@Override
	public void onDeviceConnectionError() {
		tvStatus.setText("Device couldn't be connected");
		UiHelper.getInstance().dismissProgress();
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		CharSequence items[] = new String[2];
		items[0] = "Reconnect";
		items[1] = "Auto-config";
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("Device connection error");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					DeviceEnum deviceEnum = DeviceEnum.valueOf(currentDevice.toUpperCase());
					AndroidTerminal.getInstance().initDevice(deviceEnum, DeviceConnectionType.AUDIO, null, null);
				} else if (item == 1) {
					AndroidTerminal.getInstance().startAutoConfig();
					UiHelper.getInstance().showDialog("STARTING CONFIG", getActivity(), false);
				}
				imm.hideSoftInputFromWindow(tvAmount.getWindowToken(), 0);
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}

	@Override
	public void onAutoConfigProgressUpdate(String progress) {
		tvStatus.setText("Auto config progree update");
		UiHelper.getInstance().updateMessage("AUTO CONFIG " + progress);
	}

	@Override
	public void onReversalRetrieved(CoreResponse reversalResponse) {
		tvStatus.setText("Reversal Response");
		String resp = ("Card Type :" + reversalResponse.getCardType() + "\n") + "\n" + "UniqueRef :" + reversalResponse.getUniqueRef() + "\n" + "Description :" + reversalResponse.getDescription() + "\n" + "Code :" + reversalResponse.getCode() + "\n" + "DateTime :" + reversalResponse.getDateTime() + "\n";
		UiHelper.getInstance().showSuccess(getActivity(), "Reversal Response", resp);
	}

	@Override
	public void onSelectSerialPort(ArrayList<String> ports) {

	}

	@Override
	public void onDeviceInfoReturned(HashMap<String, String> deviceInfo) {
		tvStatus.setText("Device info returned");
		String resp = "";
		String sdkVersion = AndroidTerminal.getInstance().getSdkVersion();
		resp = resp + "SDK Version : " + sdkVersion + "\n\n";
		inTransaction = false;
		cancelButton.setVisibility(View.GONE);
		for (Map.Entry<String, String> entry : deviceInfo.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			resp = resp + " " + key + ": " + value + "\n";
		}
		UiHelper.getInstance().showSuccess(getActivity(), "Device Info", resp);
	}

	@Override
	public void onButtonPressed(com.payments.core.common.enums.Button button) {
		// TODO: Not yet implemented
	}

	@Override
	public void onRequestSetAmount(CoreSale coreSale) {
		inTransaction = true;
		if (coreSale.isDelayedAuthEnabled()) {
			UiHelper.getInstance().showRequestAmountEdit(coreSale, getActivity());
		} else {
			if (tvAmount.getText().toString().length() == 0 || tvAmount.getText().toString().equals("0.00")) {
				coreSale.setAmount(BigDecimal.valueOf(1));
			} else {
				BigDecimal amount = BigDecimal.valueOf(Double.parseDouble(tvAmount.getText().toString().replaceAll("[^0-9.]", "")));
				coreSale.setAmount(amount);
			}
			AndroidTerminal.getInstance().processSale(coreSale);
		}
	}

	@Override
	public void onRequestUpdateResponse(CoreResponse response) {
		Log.d("MainActivity", "onRequestUpdateResponse ");
		if (response.getCode().equals("A")) {
			Toast.makeText(getActivity(), "Transaction update successful", Toast.LENGTH_SHORT).show();
		} else {
			Toast.makeText(getActivity(), response.getDescription(), Toast.LENGTH_SHORT).show();
		}
	}

	@Override
	public void onOfflineSaleRequest(final CoreSale saleRequest) {
		Log.d("MainActivity", "onOfflineSaleRequest " + saleRequest.getAsJsonObject());
		UiHelper.getInstance().showError(getActivity(), "Offline transaction", "Offline transaction");

		if (RestConnector.isPayconexSDK()) {
			// ----- Sample code to send the transaction online using Rest-connector directly once the internet comes back online
			final CoreTerminal coreTerminal = new CoreTerminal("220614969941", "0e8d38a08f2d4919a5a27ec83a6e8d76", "https://cert.payconex.net/api");
			coreTerminal.setCurrency(Currency.USD);
			coreTerminal.setTerminalType(TerminalType.CHP);
			new Thread(new Runnable() {
				@Override
				public void run() {
					try {
						boolean isOffline = true;
						int attempt = 0;
						while (isOffline && attempt < 5) {
							isOffline = AndroidTerminal.getInstance().offlineMode;
							if (!isOffline) {
								CoreSaleResponse response = RestConnector.sendSale(saleRequest.getAsJsonObject(), coreTerminal);
								if (response != null) {
									Log.d("MainActivity", "Successful offline sale response");
									getActivity().runOnUiThread(new Runnable() {
										public void run() {
											Toast.makeText(getActivity(), "Successful offline sale response.", Toast.LENGTH_SHORT).show();
										}
									});
								}
							}
							Thread.sleep(10000);
							attempt++;
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}).start();
		}
	}

	@Override
	public void onGiftCardDataReturned(HashMap<String, String> giftCardData) {

	}

	@Override
	public void onBalanceInquiryRetrieved(CoreResponse response) {
		if (paxDialog != null) {
			paxDialog.dismiss();
		}
		tvAmount.setText("");
		tvStatus.setText("Balance Inquiry Response");
		UiHelper.getInstance().dismissProgress();
		inTransaction = false;
		StringBuilder resp = new StringBuilder();
		for (CoreAvailableBalance balance : response.getBalanceAmount()) {
			resp.append(balance.getCardAccount().toString()).append(" ").append(balance.getBalanceAmount().toString()).append(" ").append(balance.getCurrency().toString()).append("\n");
		}
		UiHelper.getInstance().showSuccess(getActivity(), "Balance inquiry response", resp.toString());
	}

	@Override
	public void onLoyaltyCardDataRetrieved(CoreLoyaltyCardData coreLoyaltyCardData) {
		Log.d("MainActivity", "onLoyaltyCardDataRetrieved");
	}

	@Override
	public void onRequestPin(RequestPinType pinType) {
		UiHelper.getInstance().dismissProgress();
		if (pinType == RequestPinType.PIN_STARTED) {
			pinDialog = UiHelper.getInstance().showPINDialog(getActivity(), "PIN", InputType.TYPE_NULL, "",
					new UiHelper.PromptDialogCallback() {
						@Override
						public void onStart(final EditText editText) {
							pinChars = new StringBuilder();
							pinEditText = editText;
						}
					});
			pinDialog.show();
		} else if (pinType == RequestPinType.CHARACTER_ADDED) {
			pinChars.append("*");
			pinEditText.setText(pinChars.toString());
		} else if (pinType == RequestPinType.CHARACTER_REMOVED) {
			pinChars.delete(0, pinChars.length());
			pinEditText.setText(pinChars.toString());
		} else if (pinType == RequestPinType.PIN_FINISHED) {
			if (pinDialog != null) {
				pinDialog.dismiss();
			}
		}
	}

	@Override
	public void onRequestExpiryDate() {
		expiryDatePrompt = true;
		UiHelper.getInstance().dismissProgress();
		UiHelper.getInstance().showExpiryDateDialog(getActivity(),
				new UiHelper.PromptDialogCallback() {
					@Override
					public void onStart(final EditText editText) {
						AndroidTerminal.getInstance().submitExpiryDate(editText.getText().toString());
					}
				});
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		AndroidTerminal.getInstance().unRegisterCoreAPIDeviceListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPISaleListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIRefundListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIReversalListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIUpdateListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIMessageListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIReportingListener(this);
	}

	@Override
	public void onTransactionListResponse(CoreTransactions response) {

	}

	@Override
	public void onRequestCloseBatchResponse(CoreCloseBatchResponse response) {

	}

	@Override
	public void onLoginUrlRetrieved(String url) {

	}

	@Override
	public void onTransactionReportRetrieved(CoreTransactionReports response) {
		Log.d("MainActivity", "onTransactionReportRetrieved ");
		tvAmount.setText("");
		tvStatus.setText("Transaction reports");
		UiHelper.getInstance().dismissProgress();
		inTransaction = false;
		StringBuilder resp = new StringBuilder();
		resp.append("Criterion type: ").append(response.getCriterionType()).append("\n");
		resp.append("Criterion value: ").append(response.getCriterionValue()).append("\n");
		resp.append("Settlement date: ").append(response.getSettlementDate()).append("\n");
		resp.append("Balance inquiry count: ").append(response.getBalanceInquiryCount()).append("\n\n\n");
		for (CoreTransactionReportSummary summary : response.getTransactionSummary()) {
			resp.append("Purchase count: ").append(summary.getAltPurchaseCount()).append("\n");
			resp.append("Purchase total: ").append(summary.getAltPurchaseTotal()).append("\n");
			resp.append("Refund count: ").append(summary.getAltRefundCount()).append("\n");
			resp.append("Refund total: ").append(summary.getAltRefundTotal()).append("\n");
			resp.append("Card account: ").append(summary.getCardAccount()).append("\n");
			resp.append("Currency: ").append(summary.getCurrency()).append("\n");
			resp.append("Purchase count: ").append(summary.getPurchaseCount()).append("\n");
			resp.append("Purchase reversal count: ").append(summary.getPurchaseReversalCount()).append("\n");
			resp.append("Purchase reversal total: ").append(summary.getPurchaseReversalTotal()).append("\n");
			resp.append("Refund count: ").append(summary.getRefundCount()).append("\n");
			resp.append("Refund reversal count: ").append(summary.getRefundReversalCount()).append("\n");
			resp.append("Refund reversal total: ").append(summary.getRefundReversalTotal()).append("\n");
			resp.append("Refund total: ").append(summary.getRefundTotal()).append("\n");
			resp.append("Sub-total amount: ").append(summary.getSubTotal()).append("\n");
			resp.append("Sub-total count: ").append(summary.getSubTotalCount()).append("\n");
			resp.append("Withdrawal count: ").append(summary.getWithdrawalCount()).append("\n");
			resp.append("Withdrawal total: ").append(summary.getWithdrawalTotal()).append("\n\n\n");
		}
		UiHelper.getInstance().showSuccess(getActivity(), "Transaction reports response", resp.toString());
	}
}
