package com.payments.core.sdktest.activities;

import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v7.preference.ListPreference;
import android.support.v7.preference.Preference;
import android.support.v7.preference.PreferenceManager;
import android.util.Log;

import com.payments.core.admin.AndroidTerminal;
import com.payments.core.admin.TerminalBase;
import com.payments.core.common.contracts.CoreAPIDeviceListener;
import com.payments.core.common.contracts.CoreAPIMessageListener;
import com.payments.core.common.enums.Button;
import com.payments.core.common.enums.CoreDeviceError;
import com.payments.core.common.enums.CoreError;
import com.payments.core.common.enums.CoreMessage;
import com.payments.core.common.enums.CoreMode;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.LogLevel;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.common.FileDialog;
import com.payments.core.sdktest.common.UiHelper;
import com.takisoft.fix.support.v7.preference.EditTextPreference;
import com.takisoft.fix.support.v7.preference.PreferenceFragmentCompat;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class PreferenceFragment extends PreferenceFragmentCompat implements CoreAPIDeviceListener,CoreAPIMessageListener {
	SettingsListener listener;
	FileDialog fileDialog;

	public interface SettingsListener {
		void getTerminalSettings(Context context, String terminalId, String secret, CoreMode mode);
	}

	public interface CloseBatchListener {
		void onRequestCloseBatch();
	}

	public void setListener(SettingsListener listener) {
		this.listener = listener;
	}

	@Override
	public void onCreatePreferencesFix(@Nullable Bundle savedInstanceState, String rootKey) {
		PreferenceManager.setDefaultValues(getActivity(), R.xml.preferences, false);
		addPreferencesFromResource(R.xml.preferences);
		AndroidTerminal.getInstance().registerCoreAPIDeviceListener(this);
		AndroidTerminal.getInstance().registerCoreAPIMessageListener(this);
		Preference button = findPreference("saveButtonPreference");
		button.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				String terminalId;
				String secret;
				try {
					terminalId = ((EditTextPreference) findPreference("terminalIdPrefs")).getText();
					secret = ((EditTextPreference) findPreference("secretPrefs")).getText();
				} catch (Exception e) {
					terminalId = ((android.support.v7.preference.EditTextPreference) findPreference("terminalIdPrefs")).getText();
					secret = ((android.support.v7.preference.EditTextPreference) findPreference("secretPrefs")).getText();
				}
				ListPreference mode = (ListPreference) getPreferenceScreen().findPreference("modeList");
				CoreMode coreMode = CoreMode.valueOf(mode.getEntry().toString());
				listener.getTerminalSettings(getActivity(), terminalId, secret, coreMode);
				return true;
			}
		});

		Preference offlineMode = findPreference("offlineModePreference");
		offlineMode.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				UiHelper.getInstance().showOflineMode(getActivity());
				return true;
			}
		});

		Preference healthCheck = findPreference("healthCheckPreference");
		healthCheck.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				UiHelper.getInstance().showHealthCheck(getActivity(), new TerminalHealthListener());
				return true;
			}
		});

		Preference closeBatch = findPreference("closeBatch");
		closeBatch.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				UiHelper.getInstance().showCloseBatch(getActivity(), new CloseBatchListener() {
					@Override
					public void onRequestCloseBatch() {
						AndroidTerminal.getInstance().closeTransactionsBatch();
					}
				});
				return true;
			}
		});

		Preference loadFirmware = findPreference("loadFirmware");
		loadFirmware.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				DialogInterface.OnClickListener signaturePositiveAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
						AndroidTerminal.getInstance().loadFirmware();
					}
				};
				DialogInterface.OnClickListener signatureNegativeAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
					}
				};
				UiHelper.getInstance().showConfirmation(getActivity(), signaturePositiveAction, signatureNegativeAction, "Load RBA", "Are you sure ?");
				return true;
			}
		});

		Preference loadConfiguration = findPreference("loadConfiguration");
		loadConfiguration.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				DialogInterface.OnClickListener signaturePositiveAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
						if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.INGENICO)) {
							AndroidTerminal.getInstance().loadConfiguration();
						} else if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)) {
							fileDialog = new FileDialog(getActivity());
							fileDialog.addFileListener(new FileDialog.FileSelectedListener() {
								public void fileSelected(File file) {
									Log.d(getClass().getName(), "selected file " + file.toString());
									AndroidTerminal.getInstance().loadConfiguration(file.getAbsolutePath());
								}
							});
							fileDialog.showDialog();
						}
					}
				};
				DialogInterface.OnClickListener signatureNegativeAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
					}
				};
				UiHelper.getInstance().showConfirmation(getActivity(), signaturePositiveAction, signatureNegativeAction, "Load Configuration", "Are you sure ?");
				return true;
			}
		});

		Preference loadAssets = findPreference("loadAssets");
		loadAssets.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				DialogInterface.OnClickListener signaturePositiveAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
						AndroidTerminal.getInstance().loadAsset();
					}
				};
				DialogInterface.OnClickListener signatureNegativeAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
					}
				};
				UiHelper.getInstance().showConfirmation(getActivity(), signaturePositiveAction, signatureNegativeAction, "Load assets", "Are you sure ?");
				return true;
			}
		});
		Preference loadRKI = findPreference("loadRKI");
		loadRKI.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				DialogInterface.OnClickListener signaturePositiveAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
						AndroidTerminal.getInstance().loadRKI();
					}
				};
				DialogInterface.OnClickListener signatureNegativeAction = new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialogInterface, int which) {
					}
				};
				UiHelper.getInstance().showConfirmation(getActivity(), signaturePositiveAction, signatureNegativeAction, "Load RKI", "Are you sure ?");
				return true;
			}
		});

		if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.INGENICO)) {
			loadConfiguration.setVisible(false);
			loadAssets.setVisible(false);
			loadFirmware.setVisible(false);
			loadRKI.setVisible(false);
		}
		if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)) {
			loadConfiguration.setVisible(true);
		}

		ListPreference logLevelPref = (ListPreference) getPreferenceScreen().findPreference("logLevel");
		logLevelPref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
			@Override
			public boolean onPreferenceChange(Preference preference, Object newValue) {

				LogLevel logLevel = LogLevel.LEVEL_FULL;
				switch (newValue.toString()) {
					case "1": // FULL
						logLevel = LogLevel.LEVEL_FULL;
						break;
					case "2": // INFO
						logLevel = LogLevel.LEVEL_INFO;
						break;
					case "3": // ERROR
						logLevel = LogLevel.LEVEL_ERROR;
						break;
					case "4": // NONE
						logLevel = LogLevel.LEVEL_NONE;
						break;
				}

				if (AndroidTerminal.getInstance().getMode() == CoreMode.LIVE && logLevel == LogLevel.LEVEL_FULL) {
					UiHelper.getInstance().showError(getActivity(), "LIVE MODE", "LEVEL FULL NOT SUPPORTED IN LIVE MODE");
					return false;
				} else {
					preference.setDefaultValue(newValue);
					AndroidTerminal.getInstance().setLogLevel(logLevel);
					return true;
				}

			}
		});

		checkFeatureCloseBatchAvailability();
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		AndroidTerminal.getInstance().unRegisterCoreAPIDeviceListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIMessageListener(this);
	}

	public void checkFeatureCloseBatchAvailability() {
		Preference closeBatch = findPreference("closeBatch");
		Map<String, Boolean> features = AndroidTerminal.getInstance().getSettings() != null ? AndroidTerminal.getInstance().getSettings().getFeatures() : null;

		if (features != null && (!features.containsKey("enableAutomaticSettle") || !features.get("enableAutomaticSettle"))) {
			closeBatch.setVisible(true);

		} else {
			closeBatch.setVisible(false);
		}
	}

	private class TerminalHealthListener implements TerminalBase.TerminalHealthListener {

		public TerminalHealthListener() {

		}

		@Override
		public void onHealthCheckStarted() {
			Log.d("MainActivity", "Check Started ");
		}

		@Override
		public void onHealthCheckStopped() {
			Log.d("MainActivity", "Check Stoped ");
		}

		@Override
		public void onError(CoreError error) {
			Log.d("MainActivity", "error ");
		}
	}

	@Override
	public void onDeviceConnected(DeviceEnum type, HashMap<String, String> deviceInfo) {

	}


	@Override
	public void onMessage(CoreMessage message) {
		UiHelper.getInstance().showSuccess(getActivity(), "Message", message.name().replace("_", " "));
	}

	@Override
	public void onDeviceError(CoreDeviceError error, String message) {
		UiHelper.getInstance().showWarning(getActivity(), "Error", error.name());
	}

	@Override
	public void onDeviceDisconnected(DeviceEnum type) {

	}

	@Override
	public void onSelectApplication(ArrayList<String> applications) {

	}

	@Override
	public void onSelectBTDevice(ArrayList<Object> devices) {

	}

	@Override
	public void onDeviceConnectionError() {

	}

	@Override
	public void onAutoConfigProgressUpdate(String progress) {

	}

	@Override
	public void onSelectSerialPort(ArrayList<String> ports) {

	}

	@Override
	public void onDeviceInfoReturned(HashMap<String, String> deviceInfo) {

	}

	@Override
	public void onButtonPressed(Button button) {

	}

	@Override
	public void onError(CoreError error, String message) {

	}
}