package com.payments.core.sdktest.activities;

import android.content.Context;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.payments.core.CoreAvailableBalance;
import com.payments.core.CoreEmvTag;
import com.payments.core.CoreResponse;
import com.payments.core.CoreSale;
import com.payments.core.CoreSaleResponse;
import com.payments.core.CoreSignature;
import com.payments.core.CoreSurcharge;
import com.payments.core.CoreTip;
import com.payments.core.CoreUpdate;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.common.enums.Currency;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.InputMethod;
import com.payments.core.common.enums.TipType;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.adapter.CoreDetailRecyclerAdapter;
import com.payments.core.sdktest.common.SimplePojo;
import com.payments.core.sdktest.common.UiHelper;
import com.payments.core.sdktest.common.Util;
import com.payments.core.sdktest.view.CanvasView;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

public class ReceiptFragment extends Fragment implements View.OnClickListener {
	private static final String ARGUMENT_CORE_RESPONSE = "ARGUMENT_CORE_RESPONSE";
	private static final String ARGUMENT_CORE_SALE = "ARGUMENT_CORE_SALE";
	private static final String ARGUMENT_CORE_REFUND_TYPE = "ARGUMENT_CORE_REFUND_TYPE";
	private static final String ARGUMENT_CORE_REFUND_REASON = "ARGUMENT_CORE_REFUND_REASON";
	private static final String ARGUMENT_CORE_REFUND_ORDER_ID = "ARGUMENT_CORE_REFUND_ORDER_ID";
	private static final String ARGUMENT_IS_REFUND = "ARGUMENT_IS_REFUND";
	private static final String ARGUMENT_INPUT_METHOD = "ARGUMENT_INPUT_METHOD";

	private ImageView imageResult;
	private TextView textResult, textRefund, textReason;
	private Button buttonShowDetails, buttonProcessAnotherPayment, buttonUpdate, buttonPrint;
	private LinearLayout layoutRefund, layoutReason;
	private RecyclerView mRecyclerView;

	private CoreDetailRecyclerAdapter recyclerAdapter;
	private List<SimplePojo> details;
	private List<SimplePojo> simpleDetails;
	private CoreResponse coreResponse;
	private CoreSale coreSale;
	private String refundType, refundReason, refundOrderID;
	private InputMethod inputMethod;
	private boolean isRefund;
	private Animation fadeInInAnimation;
	private RelativeLayout receiptLayout;
	private CanvasView signatureCanvas;
	private RelativeLayout canvasLinearLayout;
	private Currency currency;

	public ReceiptFragment() {
		// Required empty public constructor
	}

	public static ReceiptFragment getInstance(CoreResponse coreResponse, CoreSale coreSale, InputMethod inputMethod) {
		ReceiptFragment receiptFragment = new ReceiptFragment();

		Bundle bundle = new Bundle();
		bundle.putSerializable(ARGUMENT_CORE_RESPONSE, coreResponse);
		bundle.putSerializable(ARGUMENT_CORE_SALE, coreSale);
		bundle.putSerializable(ARGUMENT_INPUT_METHOD, inputMethod);
		bundle.putBoolean(ARGUMENT_IS_REFUND, false);
		receiptFragment.setArguments(bundle);

		return receiptFragment;
	}

	public static ReceiptFragment getInstance(CoreResponse coreResponse, String refundReason, String refundType, String orderID) {
		ReceiptFragment receiptFragment = new ReceiptFragment();

		Bundle bundle = new Bundle();
		bundle.putSerializable(ARGUMENT_CORE_RESPONSE, coreResponse);
		bundle.putString(ARGUMENT_CORE_REFUND_TYPE, refundType);
		bundle.putString(ARGUMENT_CORE_REFUND_REASON, refundReason);
		bundle.putString(ARGUMENT_CORE_REFUND_ORDER_ID, orderID);
		bundle.putBoolean(ARGUMENT_IS_REFUND, true);
		receiptFragment.setArguments(bundle);

		return receiptFragment;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		currency = AndroidTerminal.getInstance().getSettings().getCurrency();
		coreResponse = (CoreResponse) getArguments().getSerializable(ARGUMENT_CORE_RESPONSE);
		isRefund = getArguments().getBoolean(ARGUMENT_IS_REFUND);

		if (isRefund) {
			refundType = getArguments().getString(ARGUMENT_CORE_REFUND_TYPE);
			refundReason = getArguments().getString(ARGUMENT_CORE_REFUND_REASON);
			refundOrderID = getArguments().getString(ARGUMENT_CORE_REFUND_ORDER_ID);

		} else {
			coreSale = (CoreSale) getArguments().getSerializable(ARGUMENT_CORE_SALE);
			inputMethod = (InputMethod) getArguments().getSerializable(ARGUMENT_INPUT_METHOD);
		}
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_receipt, container, false);

		imageResult = (ImageView) view.findViewById(R.id.receipt_image_result);
		textResult = (TextView) view.findViewById(R.id.receipt_text_result);
		textRefund = (TextView) view.findViewById(R.id.receipt_text_refund);
		textReason = (TextView) view.findViewById(R.id.receipt_text_reason);
		receiptLayout = (RelativeLayout) view.findViewById(R.id.paymentLayout);
		canvasLinearLayout = (RelativeLayout) view.findViewById(R.id.llCanvas);
		signatureCanvas = (CanvasView) view.findViewById(R.id.signature_canvas);

		layoutRefund = (LinearLayout) view.findViewById(R.id.receipt_layout_refund);
		layoutReason = (LinearLayout) view.findViewById(R.id.receipt_layout_reason);

		mRecyclerView = (RecyclerView) view.findViewById(R.id.receipt_recycle_view);
		buttonUpdate = (Button) view.findViewById(R.id.receipt_button_update);
		buttonUpdate.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				signatureCanvas.setSignature(new CoreSignature());
				receiptLayout.setVisibility(View.GONE);
				canvasLinearLayout.setVisibility(View.VISIBLE);
				canvasLinearLayout.startAnimation(fadeInInAnimation);
			}
		});
		buttonPrint = (Button) view.findViewById(R.id.receipt_button_print);
		if (!AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)) {
			buttonPrint.setVisibility(View.GONE);
		}
		buttonPrint.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				LinkedHashMap<String, Object> receipt = new LinkedHashMap<String, Object>();
				if (coreResponse.getCardHolderName() != null) {
					receipt.put("Card holder name:", coreResponse.getCardHolderName());
				}
				receipt.put("Card Type:", coreResponse.getCardType());
				receipt.put("Card number:", coreResponse.getCardNumber());
				receipt.put("Expiry date:", coreResponse.getExpiryDate());
				receipt.put("Description:", coreResponse.getDescription());
				receipt.put("Approval Code:", coreResponse.getCode());
				receipt.put("Currency:", coreResponse.getCurrency());
				receipt.put("Unique Ref:", coreResponse.getUniqueRef());
				receipt.put("Date:", Util.changeDateFormat(coreResponse.getDateTime()));
				receipt.put("Total:", AndroidTerminal.getInstance().getCurrency().getSymbol() + coreResponse.getAuthorizedAmount().toString());
				AndroidTerminal.getInstance().printReceipt(receipt, getActivity(), BitmapFactory.decodeResource(getActivity().getResources(), R.drawable.logo));
			}
		});
		if (coreResponse instanceof CoreSaleResponse) {
			buttonUpdate.setVisibility(View.VISIBLE);
		} else {
			buttonUpdate.setVisibility(View.GONE);
		}

		Button submitSignature = (Button) view.findViewById(R.id.btSignature);
		if (submitSignature != null) {
			submitSignature.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					if (signatureCanvas.checkSignature()) {
						canvasLinearLayout.setVisibility(View.GONE);
						receiptLayout.setVisibility(View.VISIBLE);
						receiptLayout.startAnimation(fadeInInAnimation);
						CoreUpdate update = new CoreUpdate(coreResponse.getUniqueRef());
						update.setSignature(signatureCanvas.getSignature());
						AndroidTerminal.getInstance().updateTransaction(update);
						signatureCanvas.clearCanvas();
					} else {
						UiHelper.getInstance().showWarning(getActivity(), "Signature empty.", "Signature cannot be empty.");
					}
				}
			});
		}

		buttonShowDetails = (Button) view.findViewById(R.id.receipt_button_show_details);
		buttonProcessAnotherPayment = (Button) view.findViewById(R.id.receipt_button_process_another_payment);
		fadeInInAnimation = AnimationUtils.loadAnimation(getActivity(),
				R.anim.fadein);
		details = new ArrayList<>();
		simpleDetails = new ArrayList<>();

		SimplePojo detail;
		String formattedAmount = Util.getFormattedAmount(coreResponse.getAuthorizedAmount().doubleValue(), currency);
		detail = new SimplePojo(R.string.transaction_detail_label_amount, currency.getSymbol() + formattedAmount);
		details.add(detail);
		simpleDetails.add(detail);
		detail = new SimplePojo(R.string.transaction_detail_label_card_number, coreResponse.getCardNumber());
		details.add(detail);
		simpleDetails.add(detail);

		if (isRefund) {
			if (refundOrderID != null && !refundOrderID.isEmpty()) {
				detail = new SimplePojo(R.string.receipt_label_order_id, refundOrderID);
				details.add(detail);
			}

			detail = new SimplePojo(R.string.receipt_label_transaction_type, getString(R.string.receipt_text_transaction_type_refund));
			details.add(detail);

		} else {
			detail = new SimplePojo(R.string.receipt_label_transaction_type, getString(R.string.receipt_text_transaction_type_card_payment));
			details.add(detail);

			if (coreSale != null) {
				if (coreSale.getTax() != null) {
					BigDecimal taxAmount = coreSale.getTax().getPercentage().multiply(coreSale.getAmount()).divide(new BigDecimal(100D), currency.getExponent(), BigDecimal.ROUND_HALF_EVEN);
					detail = new SimplePojo(R.string.receipt_label_tax_amount, currency.getSymbol() + taxAmount);
					details.add(detail);
				}

				if (coreResponse instanceof CoreSaleResponse) {
					if (((CoreSaleResponse) coreResponse).getTip() != null) {
						CoreTip coreTip = ((CoreSaleResponse) coreResponse).getTip();

						if (coreTip.getTipType() == TipType.PERCENTAGE) {
							BigDecimal tipAmount = coreTip.getPercentage().multiply(coreSale.getAmount()).divide(new BigDecimal(100D), currency.getExponent(), BigDecimal.ROUND_HALF_EVEN);
							detail = new SimplePojo(R.string.receipt_label_tip, currency.getSymbol() + tipAmount);
							details.add(detail);

						} else {
							detail = new SimplePojo(R.string.receipt_label_tip, currency.getSymbol() + coreTip.getAmount());
							details.add(detail);
						}
					}

                    if (((CoreSaleResponse) coreResponse).getSurcharge() != null) {
                        CoreSurcharge coreSurcharge = ((CoreSaleResponse) coreResponse).getSurcharge();
                        detail = new SimplePojo(R.string.receipt_label_surcharge_amount, currency.getSymbol() + coreSurcharge.getAmount());
                        details.add(detail);
                        detail = new SimplePojo(R.string.receipt_label_surcharge_percent, new DecimalFormat("#0.##").format(coreSurcharge.getPercentage()));
                        details.add(detail);
                    }
				}
			}
		}

		detail = new SimplePojo(R.string.receipt_label_name, coreResponse.getCardHolderName());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_card_type, coreResponse.getCardType());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_card_number, coreResponse.getCardNumber());
		details.add(detail);
		if (coreResponse.getExpiryDate() != null && !coreResponse.getExpiryDate().isEmpty()) {
			detail = new SimplePojo(R.string.receipt_label_expiry_date, coreResponse.getExpiryDate().substring(0, 2) + "/" + coreResponse.getExpiryDate().substring(2));
			details.add(detail);
		}

		if (isRefund) {
			detail = new SimplePojo(R.string.receipt_label_refund, refundType);
			details.add(detail);

			if (refundReason != null && !refundReason.isEmpty()) {
				detail = new SimplePojo(R.string.receipt_label_reason, refundReason);
				details.add(detail);
			}
		}

		detail = new SimplePojo(R.string.receipt_label_description, coreResponse.getDescription());
		details.add(detail);
		if (coreResponse.getApprovalCode() != null && !coreResponse.getApprovalCode().isEmpty()) {
			detail = new SimplePojo(R.string.receipt_label_approval_code, coreResponse.getApprovalCode());
			details.add(detail);
		}
		detail = new SimplePojo(R.string.receipt_label_code, coreResponse.getCode());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_currency, coreResponse.getCurrency());
		details.add(detail);

		if (coreResponse.getOrderNumber() != null) {
			detail = new SimplePojo(R.string.receipt_label_order_number, coreResponse.getOrderNumber());
			details.add(detail);
		}

		if (!isRefund) {
			if (inputMethod == null) {
				detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_emv));
				details.add(detail);
			} else {
				switch (inputMethod) {
					case INPUT_METHOD_KEYED:
						detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_keyed));
						details.add(detail);

						break;
					case INPUT_METHOD_MSR_SIGNATURE:
						detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_swipe));
						details.add(detail);

						break;
					case INPUT_METHOD_ICC:
						detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_emv));
						details.add(detail);

						break;
					case INPUT_METHOD_MSR_FALLBACK:
						detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_swipe_fallback));
						details.add(detail);

						break;
					case INPUT_METHOD_CONTACTLESS_MSR:
						detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_contactless_msr));
						details.add(detail);

						break;
					case INPUT_METHOD_CONTACTLESS_ICC:
						detail = new SimplePojo(R.string.receipt_label_input_method, getString(R.string.receipt_text_input_method_contactless_icc));
						details.add(detail);

						break;
				}
			}
		}

		detail = new SimplePojo(R.string.receipt_label_date, coreResponse.getDateTime().split("\\.")[0].replace("T", " "));
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_unique_reference, coreResponse.getUniqueRef());
		details.add(detail);
		if (coreResponse.getBalanceAmount() != null) {
			StringBuilder resp = new StringBuilder();
			for (CoreAvailableBalance balance : coreResponse.getBalanceAmount()) {
				resp.append(balance.getCardAccount().toString()).append(" ").append(balance.getBalanceAmount().toString()).append(" ").append(balance.getCurrency().toString()).append("\n");
			}
			detail = new SimplePojo(R.string.receipt_text_balance_inquiry, resp.toString());
			details.add(detail);
		}
		if (coreResponse.getCashBackAmount() != null) {
			detail = new SimplePojo(R.string.receipt_text_cashback, currency.getSymbol() + coreResponse.getCashBackAmount().toString());
			details.add(detail);
		}

		LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext()) {
			@Override
			protected int getExtraLayoutSpace(RecyclerView.State state) {
				return 1000;
			}
		};

		mRecyclerView.setLayoutManager(linearLayoutManager);
		recyclerAdapter = new CoreDetailRecyclerAdapter(getContext(), simpleDetails);
		mRecyclerView.setAdapter(recyclerAdapter);

		if (coreResponse.getCode().equals("A")) {
			imageResult.setImageResource(R.drawable.icon_approval);
			textResult.setText(R.string.receipt_text_approved);
			textResult.setTextColor(Color.parseColor("#4CAE4E"));

		} else {
			imageResult.setImageResource(R.drawable.icon_error);
			textResult.setText(R.string.receipt_text_declined);
			textResult.setTextColor(Color.parseColor("#C0202C"));
		}

		if (isRefund) {
			layoutRefund.setVisibility(View.VISIBLE);

			if (refundReason != null && !refundReason.isEmpty()) {
				layoutReason.setVisibility(View.VISIBLE);
				textReason.setText(refundReason);

			} else {
				layoutReason.setVisibility(View.GONE);
			}

			textRefund.setText(refundType);

		} else {
			layoutReason.setVisibility(View.GONE);
			layoutRefund.setVisibility(View.GONE);
		}

		if (coreResponse instanceof CoreSaleResponse) {
			CoreSaleResponse coreSaleResponse = (CoreSaleResponse) coreResponse;
			ArrayList<CoreEmvTag> listOfEMVTags = coreSaleResponse.getEmvTags();

			for (CoreEmvTag tag : listOfEMVTags) {
				if ("9F06".equals(tag.getHex().toUpperCase())) {
					detail = new SimplePojo(R.string.receipt_text_application_identifier, tag.getValue().toUpperCase());
					details.add(detail);
					break;
				}
			}
		}

		buttonShowDetails.setOnClickListener(this);
		buttonProcessAnotherPayment.setOnClickListener(this);

		buttonShowDetails.setText(R.string.receipt_button_show_details);

		return view;
	}

	@Override
	public void onAttach(Context context) {
		super.onAttach(context);
	}

	@Override
	public void onDetach() {
		super.onDetach();
	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.receipt_button_show_details:
				if (buttonShowDetails.getText().equals("Hide Details")) {
					mRecyclerView.setVisibility(View.GONE);
					mRecyclerView.startAnimation(fadeInInAnimation);
					buttonShowDetails.setText(R.string.receipt_button_show_details);
					if (isRefund) {
						layoutRefund.setVisibility(View.VISIBLE);
						if (refundReason != null && !refundReason.isEmpty()) {
							layoutReason.setVisibility(View.VISIBLE);
						}
					}
					recyclerAdapter = new CoreDetailRecyclerAdapter(getContext(), simpleDetails);
					mRecyclerView.setAdapter(recyclerAdapter);
					mRecyclerView.setVisibility(View.VISIBLE);
				} else {
					mRecyclerView.setVisibility(View.VISIBLE);
					mRecyclerView.startAnimation(fadeInInAnimation);
					layoutRefund.setVisibility(View.GONE);
					layoutReason.setVisibility(View.GONE);
					buttonShowDetails.setText(R.string.receipt_button_hide_details);
					recyclerAdapter = new CoreDetailRecyclerAdapter(getContext(), details);
					mRecyclerView.setAdapter(recyclerAdapter);
				}

				break;
			case R.id.receipt_button_process_another_payment:
				getActivity().getSupportFragmentManager().beginTransaction().remove(this).commit();
				break;
		}
	}
}
