package com.payments.core.sdktest.activities;

import android.app.DatePickerDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.InputType;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.Toast;

import com.mikepenz.fontawesome_typeface_library.FontAwesome;
import com.mikepenz.iconics.IconicsDrawable;
import com.payments.core.CoreCloseBatchResponse;
import com.payments.core.CoreRefundResponse;
import com.payments.core.CoreResponse;
import com.payments.core.CoreTransactionFilter;
import com.payments.core.CoreTransactionReports;
import com.payments.core.CoreTransactionSummary;
import com.payments.core.CoreTransactions;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.common.contracts.CoreAPIMessageListener;
import com.payments.core.common.contracts.CoreAPIRefundListener;
import com.payments.core.common.contracts.CoreAPIReportingListener;
import com.payments.core.common.contracts.CoreAPIReversalListener;
import com.payments.core.common.contracts.CoreAPIUpdateListener;
import com.payments.core.common.enums.CoreError;
import com.payments.core.common.enums.CoreMessage;
import com.payments.core.common.enums.CoreTransactionState;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.adapter.TransactionsRecyclerAdapter;
import com.payments.core.sdktest.common.UiHelper;
import com.payments.core.sdktest.view.TransactionsItemDecoration;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class ReportingFragment extends Fragment implements CoreAPIReportingListener, View.OnClickListener, View.OnTouchListener, CoreAPIRefundListener, CoreAPIReversalListener, CoreAPIUpdateListener, CoreAPIMessageListener {
	private ImageView iconFilter;
	private CheckBox checkOpen, checkClosed;
	private ImageView iconDateFrom, iconDateTo;
	private EditText editDateFrom, editDateTo;
	private Spinner spinnerItemsPerPage;
	private Button buttonFilter;
	private LinearLayout layoutFilter;
	private SwipeRefreshLayout swipeRefreshLayout;
	private RecyclerView mRecyclerView;

	private DatePickerDialog datePickerFrom, datePickerTo;
	private String formattedDateFrom, formattedDateTo;

	private TransactionsRecyclerAdapter transactionAdapter;
	private List<CoreTransactionSummary> transactionList;

	private int page;
	private boolean loading, hasMore;
	private TransactionDetailFragment transactionDetailFragment;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
							 Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_transactions,
				container, false);
		AndroidTerminal.getInstance().registerCoreAPIRefundListener(this);
		AndroidTerminal.getInstance().registerCoreAPIReversalListener(this);
		AndroidTerminal.getInstance().registerCoreAPIUpdateListener(this);
		AndroidTerminal.getInstance().registerCoreAPIMessageListener(this);
		AndroidTerminal.getInstance().registerCoreAPIReportingListener(this);

		transactionList = new ArrayList<>();

		iconFilter = (ImageView) view.findViewById(R.id.transactions_image_filter);
		checkOpen = (CheckBox) view.findViewById(R.id.transactions_filter_check_open);
		checkClosed = (CheckBox) view.findViewById(R.id.transactions_filter_check_closed);
		iconDateFrom = (ImageView) view.findViewById(R.id.transactions_filter_icon_from);
		iconDateTo = (ImageView) view.findViewById(R.id.transactions_filter_icon_to);
		editDateFrom = (EditText) view.findViewById(R.id.transactions_filter_edit_from);
		editDateTo = (EditText) view.findViewById(R.id.transactions_filter_edit_to);
		spinnerItemsPerPage = (Spinner) view.findViewById(R.id.transactions_filter_spinner_items_per_page);
		buttonFilter = (Button) view.findViewById(R.id.transactions_filter_button_filter);
		layoutFilter = (LinearLayout) view.findViewById(R.id.transactions_layout_filter);
		swipeRefreshLayout = (SwipeRefreshLayout) view.findViewById(R.id.transactions_swipe_refresh_layout);
		mRecyclerView = (RecyclerView) view.findViewById(R.id.transactions_recycle_view);

		iconFilter.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_filter).color(getResources().getColor(R.color.colorPrimary)).sizeDp(20));
		iconFilter.setOnClickListener(this);
		iconDateFrom.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_calendar_o).color(getResources().getColor(R.color.colorPrimary)).sizeDp(20));
		iconDateTo.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_calendar_o).color(getResources().getColor(R.color.colorPrimary)).sizeDp(20));
		editDateFrom.setInputType(InputType.TYPE_NULL);
		editDateFrom.setOnTouchListener(this);
		editDateTo.setInputType(InputType.TYPE_NULL);
		editDateTo.setOnTouchListener(this);
		buttonFilter.setOnClickListener(this);

		clearFilter();

		datePickerFrom = new DatePickerDialog(getContext(), new DatePickerDialog.OnDateSetListener() {
			public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
				Calendar newDate = Calendar.getInstance();
				newDate.set(year, monthOfYear, dayOfMonth);
				formattedDateFrom = new SimpleDateFormat("dd-MM-yyyy:HH:mm:ss:SSS", Locale.US).format(newDate.getTime());
				editDateFrom.setText(formattedDateFrom.split(":")[0]);
			}

		}, Calendar.getInstance().get(Calendar.YEAR), Calendar.getInstance().get(Calendar.MONTH), Calendar.getInstance().get(Calendar.DAY_OF_MONTH));
		datePickerFrom.setButton(DialogInterface.BUTTON_NEUTRAL, getString(R.string.transactions_date_picker_button_clear), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				formattedDateFrom = "";
				editDateFrom.setText("");
			}
		});

		formattedDateFrom = new SimpleDateFormat("dd-MM-yyyy:HH:mm:ss:SSS", Locale.US).format(new Date());
		formattedDateTo = new SimpleDateFormat("dd-MM-yyyy:HH:mm:ss:SSS", Locale.US).format(new Date());

		datePickerTo = new DatePickerDialog(getContext(), new DatePickerDialog.OnDateSetListener() {
			public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
				Calendar newDate = Calendar.getInstance();
				newDate.set(year, monthOfYear, dayOfMonth);
				formattedDateTo = new SimpleDateFormat("dd-MM-yyyy:HH:mm:ss:SSS", Locale.US).format(newDate.getTime());
				editDateTo.setText(formattedDateTo.split(":")[0]);
			}

		}, Calendar.getInstance().get(Calendar.YEAR), Calendar.getInstance().get(Calendar.MONTH), Calendar.getInstance().get(Calendar.DAY_OF_MONTH));
		datePickerTo.setButton(DialogInterface.BUTTON_NEUTRAL, getString(R.string.transactions_date_picker_button_clear), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				formattedDateTo = "";
				editDateTo.setText("");
			}
		});

		LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext()) {
			@Override
			protected int getExtraLayoutSpace(RecyclerView.State state) {
				return 1000;
			}
		};

		layoutFilter.setVisibility(View.GONE);
		swipeRefreshLayout.setEnabled(true);

		mRecyclerView.setLayoutManager(linearLayoutManager);
		mRecyclerView.setItemViewCacheSize(20);
		mRecyclerView.setDrawingCacheEnabled(true);
		mRecyclerView.addItemDecoration(new TransactionsItemDecoration());

		transactionAdapter = new TransactionsRecyclerAdapter(getContext(), transactionList, new TransactionsRecyclerAdapter.OnItemSelectedListener() {
			@Override
			public void onItemSelected(CoreTransactionSummary item) {
				transactionDetailFragment = TransactionDetailFragment.getInstance(item);
				getActivity().getSupportFragmentManager()
						.beginTransaction()
						.add(R.id.content_frame, transactionDetailFragment)
						.addToBackStack(null)
						.commit();
			}
		});
		mRecyclerView.setAdapter(transactionAdapter);
		mRecyclerView.addOnScrollListener(new TransactionsScrollListener(linearLayoutManager));

		swipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
			@Override
			public void onRefresh() {
				page = 0;

				clearFilter();
				requestTransactions();
			}
		});

		requestTransactions();

		return view;
	}

	@Override
	public void onMessage(CoreMessage message) {
		if (message.equals(CoreMessage.TOKEN_RETRIEVED)) {
			requestTransactions();

		} else if (message.equals(CoreMessage.TRANSACTION_UPDATE_SUCCESSFUL)) {
			UiHelper.getInstance().dismissProgress();
			if (transactionDetailFragment != null && transactionDetailFragment.isAdded()) {
				getActivity().getSupportFragmentManager().beginTransaction().remove(transactionDetailFragment).commit();
			}
		}

		Toast.makeText(getActivity(), message.name().replace("_", " "), Toast.LENGTH_SHORT).show();
	}

	@Override
	public void onRefundResponse(CoreRefundResponse response) {
		UiHelper.getInstance().dismissProgress();
		if (transactionDetailFragment != null && transactionDetailFragment.isAdded()) {
			getActivity().getSupportFragmentManager().beginTransaction().remove(transactionDetailFragment).commit();
		}
		ReceiptFragment receiptFragment = ReceiptFragment.getInstance(response, "", "REFERENCED", "");
		getActivity().getSupportFragmentManager()
				.beginTransaction()
				.add(R.id.content_frame, receiptFragment)
				.addToBackStack(null)
				.commit();
	}

	@Override
	public void onTransactionListResponse(CoreTransactions response) {
		Log.d(ReportingFragment.this.getTag(), "onTransactions ");
		transactionsReceived(response);
	}

	@Override
	public void onLoginUrlRetrieved(String url) {
		Log.d(ReportingFragment.this.getTag(), "onLoginUrlRetrieved, URL: " + url);
	}

	@Override
	public void onTransactionReportRetrieved(CoreTransactionReports response) {

	}

	@Override
	public void onError(CoreError error, String message) {
		UiHelper.getInstance().showError(getActivity(), error.name().replace("_", ""), message);
	}


	@Override
	public void onReversalRetrieved(CoreResponse reversalResponse) {
		UiHelper.getInstance().dismissProgress();
		if (transactionDetailFragment != null && transactionDetailFragment.isAdded()) {
			getActivity().getSupportFragmentManager().beginTransaction().remove(transactionDetailFragment).commit();
		}
		ReceiptFragment receiptFragment = ReceiptFragment.getInstance(reversalResponse, "", "", "");
		getActivity().getSupportFragmentManager()
				.beginTransaction()
				.add(R.id.content_frame, receiptFragment)
				.addToBackStack(null)
				.commit();
	}

	@Override
	public void onRequestUpdateResponse(CoreResponse response) {
		Log.d("MainActivity", "onRequestUpdateResponse ");
		UiHelper.getInstance().showSuccess(getActivity(), "Transaction updated", "Transaction updated successfully");
	}

	@Override
	public void onRequestCloseBatchResponse(CoreCloseBatchResponse response) {

	}

	@Override
	public void onHiddenChanged(boolean hidden) {
		super.onHiddenChanged(hidden);
		if (!hidden) {
			if (transactionList.isEmpty()) {
				page = 0;
				clearFilter();
				requestTransactions();
			}
		}
	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.transactions_image_filter:
				if (layoutFilter.getVisibility() == View.GONE) {
					layoutFilter.setVisibility(View.VISIBLE);
					swipeRefreshLayout.setEnabled(false);

				} else {
					layoutFilter.setVisibility(View.GONE);
					swipeRefreshLayout.setEnabled(true);
				}

				break;
			case R.id.transactions_filter_button_filter:
				page = 0;

				requestTransactions();
				layoutFilter.setVisibility(View.GONE);
				swipeRefreshLayout.setEnabled(true);

				break;
		}
	}

	@Override
	public boolean onTouch(View view, MotionEvent motionEvent) {
		switch (view.getId()) {
			case R.id.transactions_filter_edit_from:
				datePickerFrom.show();

				break;
			case R.id.transactions_filter_edit_to:
				datePickerTo.show();

				break;
		}

		return true;
	}

	public void transactionsReceived(CoreTransactions coreTransactions) {
		List<CoreTransactionSummary> transactionList = coreTransactions.getTransactionSummary();

		if (swipeRefreshLayout.isRefreshing()) {
			swipeRefreshLayout.setRefreshing(false);
		}

		if (transactionList == null || transactionList.isEmpty()) {
			hasMore = false;

			int count = this.transactionList.size();
			this.transactionList.clear();
			transactionAdapter.notifyItemRangeRemoved(0, count);

			loading = false;
			return;
		}

		if (this.transactionList.isEmpty()) {
			this.transactionList.addAll(transactionList);
			transactionAdapter.notifyDataSetChanged();

		} else {
			if (page == 0) {
				this.transactionList.clear();
				this.transactionList.addAll(transactionList);
				transactionAdapter.notifyDataSetChanged();
			} else {
				int start = this.transactionList.size();
				this.transactionList.addAll(transactionList);
				transactionAdapter.notifyItemRangeInserted(start, transactionList.size());
			}
		}

		page++;
		if (AndroidTerminal.getInstance().getSettings().getFeatures().containsKey("payconexSdk")) {
			hasMore = false;
		} else {
			hasMore = !(transactionList.size() < Integer.parseInt((String) spinnerItemsPerPage.getSelectedItem()));
		}
		loading = false;
	}

	private void clearFilter() {
		checkOpen.setChecked(true);
		checkClosed.setChecked(true);
		formattedDateFrom = "";
		formattedDateTo = new SimpleDateFormat("dd-MM-yyyy:HH:mm:ss:SSS", Locale.US).format(new Date());
		editDateFrom.setText("");
		editDateTo.setText("");
		spinnerItemsPerPage.setSelection(0);
	}

	private void requestTransactions() {
		CoreTransactionFilter filter = new CoreTransactionFilter();

		if (checkClosed.isChecked()) {
			if (checkOpen.isChecked()) {
				filter.setState(CoreTransactionState.ALL);

			} else {
				filter.setState(CoreTransactionState.CLOSED);
			}
		} else if (checkOpen.isChecked()) {
			filter.setState(CoreTransactionState.OPEN);

		} else {
			filter.setState(null);
		}

		filter.setResultsPerPage(Integer.parseInt((String) spinnerItemsPerPage.getSelectedItem()));
		filter.setStartDate(formattedDateFrom);
		filter.setEndDate(formattedDateTo);

		loading = true;
		AndroidTerminal.getInstance().getTransactions(page, filter);
	}

	private class TransactionsScrollListener extends RecyclerView.OnScrollListener {
		private LinearLayoutManager linearLayoutManager;

		public TransactionsScrollListener(LinearLayoutManager linearLayoutManager) {
			this.linearLayoutManager = linearLayoutManager;
		}

		@Override
		public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
			super.onScrolled(recyclerView, dx, dy);

			if (!loading) {
				if (linearLayoutManager.findLastVisibleItemPosition() == linearLayoutManager.getItemCount() - 1) {
					if (hasMore) {
						requestTransactions();
					}
				}
			}
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		AndroidTerminal.getInstance().unRegisterCoreAPIRefundListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIReversalListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIUpdateListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIMessageListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIReportingListener(this);
	}
}
