package com.payments.core.sdktest.activities;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.payments.core.CoreLoyaltyCardData;
import com.payments.core.CoreResponse;
import com.payments.core.CoreSale;
import com.payments.core.CoreSaleResponse;
import com.payments.core.CoreSecureCard;
import com.payments.core.CoreSecureCardResponse;
import com.payments.core.CoreSignature;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.common.contracts.CoreAPIDeviceListener;
import com.payments.core.common.contracts.CoreAPIMessageListener;
import com.payments.core.common.contracts.CoreAPISaleListener;
import com.payments.core.common.contracts.CoreAPISecureCardListener;
import com.payments.core.common.enums.CoreDeviceError;
import com.payments.core.common.enums.CoreError;
import com.payments.core.common.enums.CoreMessage;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.RequestPinType;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.common.UiHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;

public class SecureCardFragment extends Fragment implements CoreAPISecureCardListener, CoreAPIDeviceListener, CoreAPISaleListener, CoreAPIMessageListener {
	private Button btRegister, btUpdate, btDelete, btSale;
	private View topView;
	private EditText etMerchantReference, etSecureCardReference, editTextAmount;
	private CoreSecureCard coreSecureCard;
	private TextView tvStatus;
	private boolean didUpdate = false;
	private CardDetailsFragment cardDetailsFragment;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
							 Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_secure_card,
				container, false);
		AndroidTerminal.getInstance().registerCoreAPIDeviceListener(this);
		AndroidTerminal.getInstance().registerCoreAPISaleListener(this);
		AndroidTerminal.getInstance().registerCoreAPISecureCardListener(this);
		AndroidTerminal.getInstance().registerCoreAPIMessageListener(this);

		etMerchantReference = (EditText) view.findViewById(R.id.merchantReference);
		etSecureCardReference = (EditText) view.findViewById(R.id.secureCardReference);
		editTextAmount = (EditText) view.findViewById(R.id.editTextAmount);
		coreSecureCard = new CoreSecureCard();
		coreSecureCard.setCardReference(etSecureCardReference.getText().toString() == null ? null : etSecureCardReference.getText().toString());
		coreSecureCard.setMerchantReference(etMerchantReference.getText().toString() == null ? null : etMerchantReference.getText().toString());
		tvStatus = (TextView) view.findViewById(R.id.tvStatus);
		btRegister = (Button) view.findViewById(R.id.btRegister);
		btRegister.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (etMerchantReference.getText().length() == 0) {
					UiHelper.getInstance().showWarning(getActivity(), "Merchant reference empty", "Merchant reference cannot be empty");
					return;
				}
				didUpdate = false;
				cardDetailsFragment = CardDetailsFragment.getInstance(CardDetailsFragment.ViewType.SECURE_CARD, null, null,
						etMerchantReference.getText().toString(), etSecureCardReference.getText().toString(), CardDetailsFragment.SecureCardType.REGISTER, null);
				getActivity().getSupportFragmentManager()
						.beginTransaction()
						.add(R.id.content_frame, cardDetailsFragment)
						.addToBackStack(null)
						.commit();
			}
		});
		btUpdate = (Button) view.findViewById(R.id.btUpdate);
		btUpdate.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (etMerchantReference.getText().length() == 0) {
					UiHelper.getInstance().showWarning(getActivity(), "Merchant reference empty", "Merchant reference cannot be empty");
					return;
				}
				didUpdate = true;
				AndroidTerminal.getInstance().retrieveSecureCard(etMerchantReference.getText().toString());
			}
		});
		btDelete = (Button) view.findViewById(R.id.btDelete);
		btDelete.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				UiHelper.getInstance().showDialog("Loading", getActivity(), false);
				didUpdate = false;
				coreSecureCard = new CoreSecureCard();
				coreSecureCard.setMerchantReference(etMerchantReference.getText().toString());
				coreSecureCard.setCardReference(etSecureCardReference.getText().toString());
				AndroidTerminal.getInstance().deleteSecureCard(coreSecureCard);
			}
		});

		btSale = (Button) view.findViewById(R.id.btSale);
		btSale.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (etMerchantReference.getText().length() == 0) {
					UiHelper.getInstance().showWarning(getActivity(), "Merchant reference empty", "Merchant reference cannot be empty");
					return;
				}
				if (editTextAmount.getText().length() == 0) {
					UiHelper.getInstance().showWarning(getActivity(), "Amount empty", "Amount cannot be empty");
					return;
				}
				didUpdate = false;
				CoreSale coreSale = new CoreSale(BigDecimal.valueOf(Double.parseDouble(editTextAmount.getText().toString())));
				cardDetailsFragment = CardDetailsFragment.getInstance(CardDetailsFragment.ViewType.SECURE_CARD, coreSale, null,
						etMerchantReference.getText().toString(), etSecureCardReference.getText().toString(), CardDetailsFragment.SecureCardType.SALE, null);
				getActivity().getSupportFragmentManager()
						.beginTransaction()
						.add(R.id.content_frame, cardDetailsFragment)
						.addToBackStack(null)
						.commit();
			}
		});
		topView = (View) view.findViewById(R.id.topView);
		setColor(R.color.colorPrimary);

		return view;
	}

	private void setColor(int id) {
		topView.setBackgroundColor(getResources().getColor(id));
		btRegister.setBackgroundColor(getResources().getColor(id));
		btUpdate.setBackgroundColor(getResources().getColor(id));
		btDelete.setBackgroundColor(getResources().getColor(id));
		btSale.setBackgroundColor(getResources().getColor(id));
	}

	@Override
	public void onMessage(CoreMessage message) {
		tvStatus.setText(message.name());
		if (cardDetailsFragment != null && cardDetailsFragment.isAdded() && UiHelper.getInstance().isProgressShowing()) {
			UiHelper.getInstance().updateMessage(message.name().replace("_", " "));
		} else {
			Toast.makeText(getActivity(), message.name().replace("_", " "), Toast.LENGTH_SHORT).show();
		}
	}

	@Override
	public void onSaleResponse(CoreSaleResponse response) {
		UiHelper.getInstance().dismissProgress();
		if (cardDetailsFragment != null && cardDetailsFragment.isAdded()) {
			getActivity().getSupportFragmentManager().beginTransaction().remove(cardDetailsFragment).commit();
		}
		String resp = ("Card Type :" + response.getCardType() + "\n") + "Name :" + response.getCardHolderName() + "\n" + "UniqueRef :" + response.getUniqueRef() + "\n" + "Amount :" + response.getAuthorizedAmount() + "\n" + "Currency :" + response.getCurrency() + "\n" + "Card Number :" + response.getCardNumber() + "\n" + "Expiry Date :" + response.getExpiryDate() + "\n" + "Description :" + response.getDescription() + "\n" + "Code :" + response.getCode() + "\n" + "DateTime :" + response.getDateTime() + "\n";
		if (response.getCardReferenceNumber() != null) {
			resp += "Secure Card: " + response.getCardReferenceNumber();
			etSecureCardReference.setText(response.getCardReferenceNumber() == null ? "" : response.getCardReferenceNumber());
		}
		UiHelper.getInstance().showSuccess(getActivity(), "Sale Response", resp);
		tvStatus.setText("Sale Response");
	}


	@Override
	public void onSignatureRequired(CoreSignature signature) {

	}

	@Override
	public void onError(CoreError error, String message) {
		tvStatus.setText("Error " + error.name());
		UiHelper.getInstance().showError(getActivity(), error.name().replace("_", ""), message);
	}

	@Override
	public void onDeviceError(CoreDeviceError error, String message) {
		tvStatus.setText("Error " + error.name());
		UiHelper.getInstance().showError(getActivity(), error.name().replace("_", ""), message);
	}

	@Override
	public void onDeviceConnected(DeviceEnum type, HashMap<String, String> deviceInfo) {

	}

	@Override
	public void onDeviceDisconnected(DeviceEnum type) {

	}

	@Override
	public void onSelectApplication(ArrayList<String> applications) {

	}

	@Override
	public void onSelectBTDevice(ArrayList<Object> devices) {

	}

	@Override
	public void onDeviceConnectionError() {

	}

	@Override
	public void onAutoConfigProgressUpdate(String progress) {

	}


	@Override
	public void onSelectSerialPort(ArrayList<String> ports) {

	}

	@Override
	public void onDeviceInfoReturned(HashMap<String, String> deviceInfo) {

	}

	@Override
	public void onButtonPressed(com.payments.core.common.enums.Button button) {

	}

	@Override
	public void onSecureCardResponse(CoreSecureCardResponse secureCardResponse) {
		tvStatus.setText("Secure card response ");
		if (didUpdate) {
			didUpdate = false;
			CoreSecureCard secureCard = secureCardResponse.getCoreSecureCard();
			cardDetailsFragment = CardDetailsFragment.getInstance(CardDetailsFragment.ViewType.SECURE_CARD, null, null,
					etMerchantReference.getText().toString(), etSecureCardReference.getText().toString(), CardDetailsFragment.SecureCardType.UPDATE, secureCard);
			getActivity().getSupportFragmentManager()
					.beginTransaction()
					.add(R.id.content_frame, cardDetailsFragment)
					.addToBackStack(null)
					.commit();
		} else {
			etSecureCardReference.setText(secureCardResponse.getSecureCardReference() == null ? "" : secureCardResponse.getSecureCardReference());
			if (cardDetailsFragment != null && cardDetailsFragment.isAdded()) {
				getActivity().getSupportFragmentManager().beginTransaction().remove(cardDetailsFragment).commit();
			}
			String log = "";
			if (secureCardResponse.getSecureCardReference() != null && !secureCardResponse.getSecureCardReference().isEmpty()) {
				log += "Secure Card Reference" + secureCardResponse.getSecureCardReference() + "\n";
			}

			log += secureCardResponse.getSecureCardDescription();
			UiHelper.getInstance().showSuccess(getActivity(), "Secure Card", log);
		}
	}

	@Override
	public void onRequestSetAmount(CoreSale coreSale) {

	}

	@Override
	public void onOfflineSaleRequest(CoreSale saleRequest) {

	}

	@Override
	public void onGiftCardDataReturned(HashMap<String, String> giftCardData) {

	}

	@Override
	public void onBalanceInquiryRetrieved(CoreResponse response) {

	}

	@Override
	public void onLoyaltyCardDataRetrieved(CoreLoyaltyCardData coreLoyaltyCardData) {

	}

	@Override
	public void onRequestPin(RequestPinType pinType) {

	}

	@Override
	public void onRequestExpiryDate() {

	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		AndroidTerminal.getInstance().unRegisterCoreAPIDeviceListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPISaleListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPISecureCardListener(this);
		AndroidTerminal.getInstance().unRegisterCoreAPIMessageListener(this);
	}
}
