package com.payments.core.sdktest.activities;

import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import com.payments.core.CoreSetting;
import com.payments.core.CoreSettings;
import com.payments.core.CoreSurchargePercentage;
import com.payments.core.CoreTax;
import com.payments.core.CoreTerminalUpdate;
import com.payments.core.CoreTip;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.common.contracts.CoreAPIMessageListener;
import com.payments.core.common.enums.CoreError;
import com.payments.core.common.enums.CoreMessage;
import com.payments.core.common.enums.TipType;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.adapter.CustomListViewAdapter;
import com.payments.core.sdktest.common.RowItem;
import com.payments.core.sdktest.common.UiHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class TerminalSettingsFragment extends Fragment implements CoreAPIMessageListener {
	private Button createSettings, updateSettings, deleteSettings, getSettings;
	private View topView;
	private CoreSettings settingsRetrieved;
	private TextView tvStatus;
	private ListView listView;
	private CustomListViewAdapter adapter;

	public void setSettings(CoreSettings settings) {
		this.settingsRetrieved = settings;
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
							 Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_terminal_settings,
				container, false);
		AndroidTerminal.getInstance().registerCoreAPIMessageListener(this);

		tvStatus = (TextView) view.findViewById(R.id.tvStatus);
		getSettings = (Button) view.findViewById(R.id.btGetSettings);
		getSettings.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				UiHelper.getInstance().showDialog("Loading", getActivity(), false);
				AndroidTerminal.getInstance().retrieveSettings();
			}
		});
		createSettings = (Button) view.findViewById(R.id.btCreateSettings);
		createSettings.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				CoreTerminalUpdate terminalUpdate = new CoreTerminalUpdate();

				CoreTax tax = new CoreTax();
				tax.setPercentage(BigDecimal.valueOf(10.32));
				tax.setName("ANDROID TEST 1");

				CoreTip tip = new CoreTip();
				tip.setPercentage(BigDecimal.valueOf(14.2));
				tip.setTipType(TipType.PERCENTAGE);

				ArrayList<CoreSetting> create = new ArrayList<CoreSetting>();
				create.add(tax);
				create.add(tip);

				terminalUpdate.setSettingCreate(create);
				AndroidTerminal.getInstance().updateSettings(terminalUpdate);
				UiHelper.getInstance().showDialog("Loading", getActivity(), false);
			}
		});
		updateSettings = (Button) view.findViewById(R.id.btUpdateSettings);
		updateSettings.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				CoreTerminalUpdate terminalUpdate = new CoreTerminalUpdate();
				ArrayList<CoreSetting> update = new ArrayList<CoreSetting>();

				if (!settingsRetrieved.getTaxList().isEmpty()) {
					CoreSetting setting = settingsRetrieved.getTaxList().get(0);
					update.add(setting);
				}

				if (settingsRetrieved.getFeatures().get("allowEditSurchargesForMobile")) {
					// update default percentage
					terminalUpdate.setSurcharge(new CoreSurchargePercentage(BigDecimal.valueOf(10)));
				}

				if (!update.isEmpty()) {
					terminalUpdate.setSettingUpdate(update);
					AndroidTerminal.getInstance().updateSettings(terminalUpdate);
					UiHelper.getInstance().showDialog("Loading", getActivity(), false);
				} else if (terminalUpdate.getSurcharge() != null) {
					AndroidTerminal.getInstance().updateSettings(terminalUpdate);
					UiHelper.getInstance().showDialog("Loading", getActivity(), false);
				}
			}
		});
		deleteSettings = (Button) view.findViewById(R.id.btDeleteSettings);
		deleteSettings.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				CoreTerminalUpdate terminalUpdate = new CoreTerminalUpdate();
				ArrayList<CoreSetting> delete = new ArrayList<CoreSetting>();

				if (!settingsRetrieved.getTaxList().isEmpty()) {
					CoreSetting setting = settingsRetrieved.getTaxList().get(0);
					delete.add(setting);
				}

				if (!delete.isEmpty()) {
					terminalUpdate.setSettingDelete(delete);
					AndroidTerminal.getInstance().updateSettings(terminalUpdate);
					UiHelper.getInstance().showDialog("Loading", getActivity(), false);
				}
			}
		});
		topView = (View) view.findViewById(R.id.topView);
		setColor(R.color.colorPrimary);

		listView = (ListView) view.findViewById(R.id.listview);
		List<RowItem> rowItems = new ArrayList<>();
		adapter = new CustomListViewAdapter(getActivity(), R.layout.row, rowItems);
		adapter.clear();
		listView.setAdapter(adapter);

		return view;
	}

	private void setColor(int id) {
		topView.setBackgroundColor(getResources().getColor(id));
		getSettings.setBackgroundColor(getResources().getColor(id));
		createSettings.setBackgroundColor(getResources().getColor(id));
		updateSettings.setBackgroundColor(getResources().getColor(id));
		deleteSettings.setBackgroundColor(getResources().getColor(id));
	}

	@Override
	public void onMessage(CoreMessage message) {
		tvStatus.setText(message.name());
		UiHelper.getInstance().showWarning(getActivity(), "Message", message.name());
	}

	public void onSettingsRetrieved(CoreSettings settings) {
		tvStatus.setText("Settings retrieved ");
		UiHelper.getInstance().dismissProgress();
		Drawable icon = getResources().getDrawable(R.drawable.settings);
		adapter.clear();
		if (settings.getFeatures() != null) {
			for (Map.Entry<String, Boolean> entry : settings.getFeatures().entrySet()) {
				adapter.add(new RowItem(icon, "Feature", "", entry.getKey(), entry.getValue().toString()));
			}
		}

		if (settings.getMerchantDetails() != null) {
			adapter.add(new RowItem(icon, "Name", "", settings.getMerchantDetails().getName(), ""));
			adapter.add(new RowItem(icon, "Address", "", settings.getMerchantDetails().getAddress1(), ""));
			adapter.add(new RowItem(icon, "Country", "", settings.getMerchantDetails().getCountry(), ""));
			adapter.add(new RowItem(icon, "City", "", settings.getMerchantDetails().getCity(), ""));
			adapter.add(new RowItem(icon, "Phone", "", settings.getMerchantDetails().getPhone(), ""));
		}

		if (settings.getTaxList() != null && settings.getTipList() != null) {
			ArrayList<CoreSetting> tax = settings.getTaxList();
			for (CoreSetting taxSetting : tax) {
				CoreTax tr = (CoreTax) taxSetting;
				adapter.add(new RowItem(icon, "Name", tr.getName(), tr.getPercentage().toString(), tr.getUid()));
			}
			ArrayList<CoreSetting> tip = settings.getTipList();
			for (CoreSetting tipSetting : tip) {
				if (tipSetting == null) {
					continue;
				}
				CoreTip tr = (CoreTip) tipSetting;
				adapter.add(new RowItem(icon, tr.getUid(), tr.getTipType() != null ? tr.getTipType().toString() : "", tr.getPercentage() != null ? tr.getPercentage().toString() : "", tr.getAmount() != null ? tr.getAmount().toString() : ""));
			}
		}
	}

	@Override
	public void onError(CoreError error, String message) {

	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		AndroidTerminal.getInstance().unRegisterCoreAPIMessageListener(this);
	}
}