package com.payments.core.sdktest.activities;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.TextView;

import com.mikepenz.fontawesome_typeface_library.FontAwesome;
import com.mikepenz.iconics.IconicsDrawable;
import com.payments.core.CoreRefund;
import com.payments.core.CoreReversal;
import com.payments.core.CoreReversalEbt;
import com.payments.core.CoreSettings;
import com.payments.core.CoreTip;
import com.payments.core.CoreTransactionSummary;
import com.payments.core.CoreUpdate;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.common.enums.Currency;
import com.payments.core.common.enums.DeviceEnum;
import com.payments.core.common.enums.TipType;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.adapter.CoreDetailRecyclerAdapter;
import com.payments.core.sdktest.common.SimplePojo;
import com.payments.core.sdktest.common.UiHelper;
import com.payments.core.sdktest.common.Util;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import cn.pedant.SweetAlert.SweetAlertDialog;

public class TransactionDetailFragment extends Fragment implements View.OnClickListener, View.OnFocusChangeListener {

	private static final String ARGUMENT_TRANSACTION = "ARGUMENT_TRANSACTION";

	private RecyclerView mRecyclerView;
	private EditText editRefundAmount, editRefundReason, editTipAdjustmentValue;
	private TextView labelRefundReason;
	private ImageView iconRefundReason;
	private LinearLayout layoutRefund, layoutButtons, layoutTipAdjustment;
	private RadioButton radioTipFixed, radioTipPercentage;
	private Button buttonRefund, buttonVoid, buttonSubmitRefund, buttonCancelRefund, buttonTipAdjustment, buttonSubmitTipAdjustment, buttonCancelTipAdjustment, buttonReversal;

	private CoreDetailRecyclerAdapter recyclerAdapter;
	private List<SimplePojo> details;
	private CoreTransactionSummary transaction;
	private CoreSettings settings;

	public TransactionDetailFragment() {
		// Required empty public constructor
	}

	public static TransactionDetailFragment getInstance(CoreTransactionSummary transaction) {
		TransactionDetailFragment tipFragment = new TransactionDetailFragment();

		Bundle bundle = new Bundle();
		bundle.putSerializable(ARGUMENT_TRANSACTION, transaction);
		tipFragment.setArguments(bundle);

		return tipFragment;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		transaction = (CoreTransactionSummary) getArguments().getSerializable(ARGUMENT_TRANSACTION);
		settings = AndroidTerminal.getInstance().getSettings();
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_transaction_detail, container, false);

		mRecyclerView = (RecyclerView) view.findViewById(R.id.transaction_detail_recycle_view);
		editRefundAmount = (EditText) view.findViewById(R.id.transaction_detail_edit_amount);
		editRefundReason = (EditText) view.findViewById(R.id.transaction_detail_edit_reason);
		labelRefundReason = (TextView) view.findViewById(R.id.transaction_detail_label_reason);
		iconRefundReason = (ImageView) view.findViewById(R.id.transaction_detail_icon_reason);
		layoutRefund = (LinearLayout) view.findViewById(R.id.transaction_detail_layout_refund);
		layoutTipAdjustment = (LinearLayout) view.findViewById(R.id.transaction_detail_layout_tip_adjustment);
		layoutButtons = (LinearLayout) view.findViewById(R.id.transaction_detail_layout_buttons);
		buttonSubmitRefund = (Button) view.findViewById(R.id.transaction_detail_button_submit_refund);
		buttonCancelRefund = (Button) view.findViewById(R.id.transaction_detail_button_cancel_refund);
		buttonRefund = (Button) view.findViewById(R.id.transaction_detail_button_refund);
		buttonVoid = (Button) view.findViewById(R.id.transaction_detail_button_void);
		editTipAdjustmentValue = (EditText) view.findViewById(R.id.transaction_detail_tip_adjustment_amount);
		radioTipFixed = (RadioButton) view.findViewById(R.id.transaction_detail_radio_fixed_amount);
		radioTipPercentage = (RadioButton) view.findViewById(R.id.transaction_detail_radio_percentage);
		buttonTipAdjustment = (Button) view.findViewById(R.id.transaction_detail_button_tip_adjustment);
		buttonSubmitTipAdjustment = (Button) view.findViewById(R.id.transaction_detail_button_submit_tip_adjustment);
		buttonCancelTipAdjustment = (Button) view.findViewById(R.id.transaction_detail_button_cancel_tip_adjustment);
		buttonReversal = (Button) view.findViewById(R.id.transaction_detail_button_reversal);

		editRefundAmount.setHint(settings.getCurrency().getSymbol() + (new BigDecimal(0)).setScale(settings.getCurrency().getExponent(), RoundingMode.HALF_EVEN).toString());
		editRefundAmount.addTextChangedListener(Util.addCurrencyMask(settings.getCurrency(), null, editRefundAmount));

		editRefundReason.setOnFocusChangeListener(this);
		iconRefundReason.setImageDrawable(new IconicsDrawable(getActivity()).icon(FontAwesome.Icon.faw_comment).color(getResources().getColor(R.color.colorPrimary)).sizeDp(25));

		layoutRefund.setVisibility(View.GONE);
		buttonSubmitRefund.setOnClickListener(this);
		buttonCancelRefund.setOnClickListener(this);

		layoutTipAdjustment.setVisibility(View.GONE);
		buttonSubmitTipAdjustment.setOnClickListener(this);
		buttonCancelTipAdjustment.setOnClickListener(this);

		buttonRefund.setOnClickListener(this);
		buttonRefund.setVisibility(View.GONE);
		buttonRefund.setBackgroundColor(getResources().getColor(R.color.colorPrimary));

		buttonReversal.setOnClickListener(this);
		buttonReversal.setVisibility(View.GONE);
		buttonReversal.setBackgroundColor(getResources().getColor(R.color.colorPrimary));

		buttonTipAdjustment.setOnClickListener(this);
		buttonTipAdjustment.setVisibility(View.GONE);
		buttonTipAdjustment.setBackgroundColor(getResources().getColor(R.color.colorPrimary));

		buttonVoid.setOnClickListener(this);
		buttonVoid.setVisibility(View.GONE);
		buttonVoid.setBackgroundColor(getResources().getColor(R.color.colorPrimary));


		if (transaction.getTransactionType().equals("SALE")) {
			if (transaction.getTransactionStatus().equals("READY") || transaction.getTransactionStatus().equals("PENDING") || transaction.getTransactionStatus().equals("COMPLETE") ||
					transaction.getTransactionStatus().equals("APPROVED")) {
				if (transaction.getTransactionState() == null) {
					buttonVoid.setVisibility(View.VISIBLE);
					buttonRefund.setVisibility(View.VISIBLE);
					buttonReversal.setVisibility(View.VISIBLE);
				} else {
					if (!transaction.getTransactionState().equals("CLOSED")) {
						buttonVoid.setVisibility(View.VISIBLE);
						if (!transaction.getTransactionStatus().equals("APPROVED")) {
							buttonTipAdjustment.setVisibility(View.VISIBLE);
						}
					}
					buttonRefund.setVisibility(View.VISIBLE);
					buttonReversal.setVisibility(View.VISIBLE);
				}
			}
		}

		details = new ArrayList<>();
		SimplePojo detail;

		String formattedAmount = Util.getFormattedAmount(transaction.getAmount().doubleValue(), settings.getCurrency());
		detail = new SimplePojo(R.string.transaction_detail_label_amount, settings.getCurrency().getSymbol() + formattedAmount);
		details.add(detail);

		if (transaction.getSurcharge() != null) {
			String formattedSurchargeAmount = Util.getFormattedAmount(transaction.getSurcharge().getAmount().doubleValue(), transaction.getSurcharge().getCurrency());
			detail = new SimplePojo(R.string.transaction_detail_label_surcharge_amount, transaction.getSurcharge().getCurrency().getSymbol() + formattedSurchargeAmount);
			details.add(detail);
			detail = new SimplePojo(R.string.transaction_detail_label_surcharge_percent, transaction.getSurcharge().getPercentage().setScale(2, RoundingMode.CEILING) + "%");
			details.add(detail);
		}

		detail = new SimplePojo(R.string.transaction_detail_label_name, transaction.getCardHolderName());
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_card_number, transaction.getCardNumber());
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_card_type, transaction.getCardType());
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_currency, transaction.getCurrency());
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_order_id, transaction.getOrderId());
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_transaction_date, transaction.getTransactionDate().split("\\.")[0].replace("T", " "));
		details.add(detail);
		if (transaction.getTransactionState() != null) {
			detail = new SimplePojo(R.string.transaction_detail_label_transaction_state, transaction.getTransactionState());
			details.add(detail);
		}

		detail = new SimplePojo(R.string.transaction_detail_label_transaction_status, transaction.getTransactionStatus()); //TODO CHANGE COLOR
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_transaction_type, transaction.getTransactionType());
		details.add(detail);

		detail = new SimplePojo(R.string.transaction_detail_label_unique_ref, transaction.getUniqueRef());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_code, transaction.getCode());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_approval_code, transaction.getApprovalCode());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_description, transaction.getDescription());
		details.add(detail);
		detail = new SimplePojo(R.string.receipt_label_expiry_date, transaction.getExpiryDate());
		details.add(detail);

		recyclerAdapter = new CoreDetailRecyclerAdapter(getContext(), details);
		LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext()) {
			@Override
			protected int getExtraLayoutSpace(RecyclerView.State state) {
				return 1000;
			}
		};
		mRecyclerView.setLayoutManager(linearLayoutManager);
		mRecyclerView.setAdapter(recyclerAdapter);

		return view;
	}

	@Override
	public void onAttach(Context context) {
		super.onAttach(context);
	}

	@Override
	public void onDetach() {
		super.onDetach();
	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.transaction_detail_button_tip_adjustment:
				layoutTipAdjustment.setVisibility(View.VISIBLE);
				layoutButtons.setVisibility(View.GONE);

				break;
			case R.id.transaction_detail_button_cancel_tip_adjustment:
				layoutTipAdjustment.setVisibility(View.GONE);
				layoutButtons.setVisibility(View.VISIBLE);

				break;
			case R.id.transaction_detail_button_reversal:
				if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)){
					CoreReversalEbt reversal = new CoreReversalEbt("PAX_A920_ATTENDED", transaction.getUniqueRef(), "reason");
					reversal.setDeviceId("PAX_A920_ATTENDED");
					reversal.setOperator("android_operator");
					AndroidTerminal.getInstance().processReversal(reversal);
				} else {
					CoreReversal reversal = new CoreReversal("NOMAD", transaction.getUniqueRef(), "reason");
					AndroidTerminal.getInstance().processReversal(reversal);
				}
				UiHelper.getInstance().showDialog("Loading...", getActivity(), false);
				break;
			case R.id.transaction_detail_button_submit_tip_adjustment:
				if (editTipAdjustmentValue.getText().toString().isEmpty()) {
					UiHelper.getInstance().showError(getActivity(), "Invalid tip", "Invalid tip adjustment");
					return;
				}
				CoreUpdate coreUpdate;
				CoreTip tipAdjustment = new CoreTip();
				BigDecimal value = new BigDecimal(editTipAdjustmentValue.getText().toString().replaceAll("[^0-9.]", ""));

				if (radioTipFixed.isChecked()) {
					tipAdjustment.setTipType(TipType.FIXED_AMOUNT);
					tipAdjustment.setAmount(value);

				} else {
					tipAdjustment.setTipType(TipType.PERCENTAGE);
					tipAdjustment.setPercentage(value);

					BigDecimal tipAmount = transaction.getAmount().multiply(value).divide(new BigDecimal(100), Currency.valueOf(transaction.getCurrency()).getExponent(), BigDecimal.ROUND_HALF_EVEN);
					tipAdjustment.setAmount(tipAmount);
				}

				coreUpdate = new CoreUpdate(transaction.getUniqueRef(), null, null, tipAdjustment);

				AndroidTerminal.getInstance().updateTransaction(coreUpdate);
				UiHelper.getInstance().showDialog("Loading...", getActivity(), false);

				break;
			case R.id.transaction_detail_button_refund:
				layoutRefund.setVisibility(View.VISIBLE);
				layoutButtons.setVisibility(View.GONE);

				break;
			case R.id.transaction_detail_button_cancel_refund:
				layoutRefund.setVisibility(View.GONE);
				layoutButtons.setVisibility(View.VISIBLE);

				break;
			case R.id.transaction_detail_button_submit_refund:
				if (editRefundAmount.getText().toString().isEmpty()) {
					UiHelper.getInstance().showError(getActivity(), "Invalid amount", "Invalid amount");
					return;
				}

				if (editRefundReason.getText().toString().isEmpty()) {
					UiHelper.getInstance().showError(getActivity(), "Invalid reason", "Invalid reason");
					return;
				}

				BigDecimal amount = new BigDecimal(editRefundAmount.getText().toString().replaceAll("[^0-9.]", ""));

				if (amount.doubleValue() > transaction.getAmount().doubleValue()) {
					UiHelper.getInstance().showError(getActivity(), "Invalid amount", "Invalid amount");
					return;
				}

				CoreRefund refund = new CoreRefund(amount);
				refund.setUniqueRef(transaction.getUniqueRef());
				refund.setReason(editRefundReason.getText().toString());
				AndroidTerminal.getInstance().processRefund(refund);
				UiHelper.getInstance().showDialog("Loading...", getActivity(), false);
				break;
			case R.id.transaction_detail_button_void:
				new SweetAlertDialog(getActivity(), SweetAlertDialog.WARNING_TYPE)
						.setTitleText("Void the transaction")
						.setContentText("Would you like to void the transaction ?")
						.setConfirmClickListener(new SweetAlertDialog.OnSweetClickListener() {
							@Override
							public void onClick(SweetAlertDialog sDialog) {
								sDialog.dismissWithAnimation();
								CoreRefund refund = new CoreRefund(transaction.getAmount());
								refund.setUniqueRef(transaction.getUniqueRef());
								refund.setReason("reason");
								if (AndroidTerminal.getInstance().getDevice().equals(DeviceEnum.PAX)){
									refund.setDeviceId("PAX_A920_ATTENDED");
									refund.setOperator("android_operator");
								}
								AndroidTerminal.getInstance().processRefund(refund);
								UiHelper.getInstance().showDialog("Loading...", getActivity(), false);
							}
						})
						.show();
				break;
		}
	}

	@Override
	public void onFocusChange(View view, boolean focused) {
		if (focused) {
			labelRefundReason.setScaleX(0.7F);
			labelRefundReason.setScaleY(0.7F);

		} else {
			if (editRefundReason.getText().toString().isEmpty()) {
				labelRefundReason.setScaleX(1);
				labelRefundReason.setScaleY(1);

			} else {
				labelRefundReason.setScaleX(0.7F);
				labelRefundReason.setScaleY(0.7F);
			}
		}
	}

}
