package com.payments.core.sdktest.adapter;

import android.content.Context;
import android.graphics.Color;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.mikepenz.fontawesome_typeface_library.FontAwesome;
import com.mikepenz.iconics.IconicsDrawable;
import com.payments.core.CoreTransactionSummary;
import com.payments.core.sdktest.R;

import java.util.List;


public class TransactionsRecyclerAdapter extends RecyclerView.Adapter<TransactionsRecyclerAdapter.TransactionViewHolder> {
	private Context context;
	private List<CoreTransactionSummary> transactionList;
	private OnItemSelectedListener listener;

	public TransactionsRecyclerAdapter(Context context, List<CoreTransactionSummary> transactionList, OnItemSelectedListener listener) {
		this.context = context;
		this.transactionList = transactionList;
		this.listener = listener;
	}

	@Override
	public TransactionViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
		return new TransactionViewHolder(LayoutInflater.from(context).inflate(R.layout.item_transaction, null));
	}

	@Override
	public void onBindViewHolder(TransactionViewHolder holder, int position) {
		final CoreTransactionSummary transaction = transactionList.get(position);

		if (transaction.getTransactionState() == null) {
			if (transaction.getTransactionType().equals("SALE") && transaction.getTransactionStatus().equals("APPROVED")) {
				holder.layoutStatus.setBackgroundColor(context.getResources().getColor(R.color.transactions_background_status_complete));
			} else {
				holder.layoutStatus.setBackgroundColor(context.getResources().getColor(R.color.transactions_background_status_rejected));
			}
		} else {
			if (transaction.getTransactionType().equals("SALE") && !transaction.getTransactionStatus().equals("READY") &&
					!transaction.getTransactionStatus().equals("PENDING") && !transaction.getTransactionStatus().equals("COMPLETE")) {
				holder.layoutStatus.setBackgroundColor(context.getResources().getColor(R.color.transactions_background_status_rejected));
			} else {
				if (transaction.getTransactionStatus().equals("READY")) {
					holder.layoutStatus.setBackgroundColor(context.getResources().getColor(R.color.transactions_background_status_ready));
				} else if (transaction.getTransactionStatus().equals("PENDING")) {
					holder.layoutStatus.setBackgroundColor(context.getResources().getColor(R.color.transactions_background_status_pending));
				} else {
					holder.layoutStatus.setBackgroundColor(context.getResources().getColor(R.color.transactions_background_status_complete));
				}
			}
		}

		holder.iconStatus.setImageDrawable(new IconicsDrawable(context).icon(FontAwesome.Icon.faw_check_circle_o).color(Color.WHITE).sizeDp(15));
		holder.iconArrow.setImageDrawable(new IconicsDrawable(context).icon(FontAwesome.Icon.faw_angle_right).color(Color.DKGRAY).sizeDp(20));
		holder.transactionDate.setText(transaction.getTransactionDate().replace("T", " "));
		holder.transactionValue.setText(transaction.getCurrency() + transaction.getAmount());
		holder.cardHolderName.setText(transaction.getCardHolderName());
		holder.cardNumber.setText(transaction.getCardNumber());
		holder.itemView.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				listener.onItemSelected(transaction);
			}
		});
	}

	@Override
	public int getItemCount() {
		return transactionList.size();
	}

	protected class TransactionViewHolder extends RecyclerView.ViewHolder {
		private RelativeLayout layoutStatus;
		private ImageView iconStatus, iconArrow;
		private TextView transactionDate, transactionValue, cardHolderName, cardNumber;
		private View itemView;

		public TransactionViewHolder(View itemView) {
			super(itemView);

			layoutStatus = (RelativeLayout) itemView.findViewById(R.id.item_transaction_layout_status);
			iconStatus = (ImageView) itemView.findViewById(R.id.item_transaction_icon_status);
			iconArrow = (ImageView) itemView.findViewById(R.id.item_transaction_icon_arrow);
			transactionDate = (TextView) itemView.findViewById(R.id.item_transaction_text_date);
			transactionValue = (TextView) itemView.findViewById(R.id.item_transaction_text_value);
			cardHolderName = (TextView) itemView.findViewById(R.id.item_transaction_text_card_name);
			cardNumber = (TextView) itemView.findViewById(R.id.item_transaction_text_card_number);
			this.itemView = itemView;
		}
	}


	public interface OnItemSelectedListener {
		void onItemSelected(CoreTransactionSummary item);
	}
}
