package com.payments.core.sdktest.common;


import android.app.Activity;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.widget.CompoundButton;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.BooleanResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.wallet.Cart;
import com.google.android.gms.wallet.FullWallet;
import com.google.android.gms.wallet.FullWalletRequest;
import com.google.android.gms.wallet.LineItem;
import com.google.android.gms.wallet.MaskedWallet;
import com.google.android.gms.wallet.MaskedWalletRequest;
import com.google.android.gms.wallet.PaymentMethodToken;
import com.google.android.gms.wallet.PaymentMethodTokenizationParameters;
import com.google.android.gms.wallet.PaymentMethodTokenizationType;
import com.google.android.gms.wallet.Wallet;
import com.google.android.gms.wallet.WalletConstants;
import com.google.android.gms.wallet.fragment.SupportWalletFragment;
import com.google.android.gms.wallet.fragment.WalletFragmentInitParams;
import com.google.android.gms.wallet.fragment.WalletFragmentMode;
import com.google.android.gms.wallet.fragment.WalletFragmentOptions;
import com.google.android.gms.wallet.fragment.WalletFragmentStyle;
import com.payments.core.CoreAndroidWallet;
import com.payments.core.CoreUtil;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.sdktest.R;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static android.app.Activity.RESULT_OK;


public class AndroidPay implements CompoundButton.OnCheckedChangeListener,
		GoogleApiClient.OnConnectionFailedListener {
	private final String TAG = "AndroidPayActivity";
	private static final int WALLET_ENVIRONMENT = WalletConstants.ENVIRONMENT_TEST;
	private static final String CURRENCY_CODE_USD = "USD";
	private static final int REQUEST_CODE_MASKED_WALLET = 1001;
	private static final int REQUEST_CODE_RESOLVE_LOAD_FULL_WALLET = 1004;
	private GoogleApiClient mGoogleApiClient;
	private String amount;
	private FragmentActivity mainActivity;
	private static AndroidPay singleton = null;

	public static AndroidPay getInstance(FragmentActivity mainActivity) {
		if (singleton == null) {
			singleton = new AndroidPay(mainActivity);
		}
		return singleton;
	}

	public AndroidPay(FragmentActivity mainActivity) {
		this.mainActivity = mainActivity;
		mGoogleApiClient = new GoogleApiClient.Builder(mainActivity)
				.addApi(Wallet.API, new Wallet.WalletOptions.Builder()
						.setEnvironment(WALLET_ENVIRONMENT)
						.build())
				.enableAutoManage(mainActivity, this)
				.build();
	}

	public void setUp(String amount) {
		this.amount = amount;
		createWallet();
	}

	@Override
	public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
		Log.e(TAG, "onCheckedChanged:");
	}

	@Override
	public void onConnectionFailed(@NonNull ConnectionResult connectionResult) {
		Log.e(TAG, "onConnectionFailed:");
		Toast.makeText(mainActivity, "Google Play Services error", Toast.LENGTH_SHORT).show();
	}

	public void createWallet() {
		Wallet.Payments.isReadyToPay(mGoogleApiClient).setResultCallback(
				new ResultCallback<BooleanResult>() {
					@Override
					public void onResult(@NonNull BooleanResult booleanResult) {
						if (booleanResult.getStatus().isSuccess()) {
							if (booleanResult.getValue()) {
								WalletFragmentStyle walletFragmentStyle = new WalletFragmentStyle()
										.setBuyButtonText(WalletFragmentStyle.BuyButtonText.BUY_WITH)
										.setBuyButtonAppearance(WalletFragmentStyle.BuyButtonAppearance.ANDROID_PAY_DARK)
										.setBuyButtonWidth(WalletFragmentStyle.Dimension.MATCH_PARENT);

								PaymentMethodTokenizationParameters parameters = PaymentMethodTokenizationParameters.newBuilder()
										.setPaymentMethodTokenizationType(PaymentMethodTokenizationType.NETWORK_TOKEN)
										.addParameter("publicKey", mainActivity.getString(R.string.public_key))
										.build();

								WalletFragmentOptions options = WalletFragmentOptions.newBuilder()
										.setEnvironment(WALLET_ENVIRONMENT)
										.setTheme(WalletConstants.THEME_LIGHT)
										.setFragmentStyle(walletFragmentStyle)
										.setMode(WalletFragmentMode.BUY_BUTTON)
										.build();
								List<LineItem> list = buildLineItems();
								SupportWalletFragment mWalletFragment = SupportWalletFragment.newInstance(options);
								MaskedWalletRequest walletRequest = MaskedWalletRequest.newBuilder()
										.setMerchantName("Sample App")
										.setCurrencyCode(CURRENCY_CODE_USD)
										.setEstimatedTotalPrice(amount)
										.setPaymentMethodTokenizationParameters(parameters)
										.setCart(Cart.newBuilder()
												.setCurrencyCode(CURRENCY_CODE_USD)
												.setTotalPrice(amount)
												.setLineItems(list)
												.build())
										.build();
								WalletFragmentInitParams startParams = WalletFragmentInitParams.newBuilder()
										.setMaskedWalletRequest(walletRequest)
										.setMaskedWalletRequestCode(REQUEST_CODE_MASKED_WALLET)
										.build();

								mWalletFragment.initialize(startParams);
								Wallet.Payments.loadMaskedWallet(mGoogleApiClient, walletRequest, REQUEST_CODE_MASKED_WALLET);
							} else {
								Toast.makeText(mainActivity, "Android pay cannot be used.", Toast.LENGTH_SHORT).show();
							}
						} else {
							Toast.makeText(mainActivity, "Android pay cannot be used.", Toast.LENGTH_SHORT).show();
						}
					}
				});
	}

	private void requestFullWallet(MaskedWallet maskedWallet) {
		List<LineItem> list = buildLineItems();
		FullWalletRequest fullWalletRequest = FullWalletRequest.newBuilder()
				.setGoogleTransactionId(maskedWallet.getGoogleTransactionId())
				.setCart(Cart.newBuilder()
						.setCurrencyCode(CURRENCY_CODE_USD)
						.setTotalPrice(amount)
						.setLineItems(list)
						.build())
				.build();

		Wallet.Payments.loadFullWallet(mGoogleApiClient, fullWalletRequest,
				REQUEST_CODE_RESOLVE_LOAD_FULL_WALLET);
	}

	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
			case REQUEST_CODE_MASKED_WALLET:
				switch (resultCode) {
					case RESULT_OK:
						if (data != null) {
							MaskedWallet maskedWallet =
									data.getParcelableExtra(WalletConstants.EXTRA_MASKED_WALLET);
							requestFullWallet(maskedWallet);
						}
						break;
					case Activity.RESULT_CANCELED:
						break;
					default:
						Toast.makeText(mainActivity, "Android pay error occurred.", Toast.LENGTH_SHORT).show();
						break;
				}
				break;
			case REQUEST_CODE_RESOLVE_LOAD_FULL_WALLET:
				switch (resultCode) {
					case RESULT_OK:
						if (data != null && data.hasExtra(WalletConstants.EXTRA_FULL_WALLET)) {
							FullWallet fullWallet = data.getParcelableExtra(WalletConstants.EXTRA_FULL_WALLET);
							processAndroidSale(fullWallet);
						} else if (data != null && data.hasExtra(WalletConstants.EXTRA_MASKED_WALLET)) {
							requestFullWallet((MaskedWallet) data.getParcelableExtra(WalletConstants.EXTRA_MASKED_WALLET));
						}
						break;
					case Activity.RESULT_CANCELED:
						break;
					default:
						Toast.makeText(mainActivity, "Android pay error occurred.", Toast.LENGTH_SHORT).show();
						break;
				}
				break;

			case WalletConstants.RESULT_ERROR:
				Toast.makeText(mainActivity, "Android pay error occurred.", Toast.LENGTH_SHORT).show();
				break;
		}
	}

	private void processAndroidSale(FullWallet fullWallet) {
		PaymentMethodToken token = fullWallet.getPaymentMethodToken();
		if (token != null) {
			String tokenString = CoreUtil.convertAsciiToHex(token.getToken());
			CoreAndroidWallet wallet = new CoreAndroidWallet(BigDecimal.valueOf(Double.parseDouble(amount)), tokenString, "Test", "ANDROIDPAY");
			AndroidTerminal.getInstance().processSale(wallet);
		}
	}

	private List<LineItem> buildLineItems() {
		List<LineItem> list = new ArrayList<>();
		list.add(LineItem.newBuilder()
				.setCurrencyCode(CURRENCY_CODE_USD)
				.setDescription("Test")
				.setQuantity("1")
				.setUnitPrice(amount)
				.setTotalPrice(amount)
				.build());
		return list;
	}
}


