package com.payments.core.sdktest.common;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Environment;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class FileDialog {
	private String[] fileList;
	private File currentPath;
	private FileSelectedListener fileListener;
	private final Activity activity;

	public interface FileSelectedListener {
		void fileSelected(File file);
	}

	public FileDialog(Activity activity) {
		this.activity = activity;
		loadFileList();
	}

	public void addFileListener(FileSelectedListener listener) {
		fileListener = listener;
	}

	public void showDialog() {
		AlertDialog.Builder builder = new AlertDialog.Builder(activity);
		builder.setTitle(currentPath.getPath());
		builder.setItems(fileList, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				String fileChosen = fileList[which];
				File chosenFile = new File(currentPath, fileChosen);
				fileListener.fileSelected(chosenFile);
			}
		});
		builder.show();
	}

	private void loadFileList() {
		this.currentPath = new File(Environment.getExternalStorageDirectory().toString());
		List<String> r = new ArrayList<>();
		if (currentPath.exists()) {
			FilenameFilter filter = new FilenameFilter() {
				public boolean accept(File dir, String filename) {
					File sel = new File(dir, filename);
					if (!sel.canRead()) return false;
					else {
						return filename.toLowerCase().endsWith(".zip");
					}
				}
			};
			String[] fileList1 = currentPath.list(filter);
			Collections.addAll(r, fileList1);
		}
		fileList = r.toArray(new String[]{});
	}
}
