package com.payments.core.sdktest.common;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.payments.core.admin.AndroidTerminal;
import com.payments.core.sdktest.R;

public class InputAccountDialog extends Dialog {

	private EditText cardNumEdt;
	private TextView inputType;
	private String amount, inputTypeText;
	private DialogCallback dialogCallback;

	public InputAccountDialog(@NonNull Context context, String amount, String inputTypeText, DialogCallback dialogCallback) {
		super(context);
		init(amount, inputTypeText, dialogCallback);
	}

	private void init(String amountSale, String inputTypeText, DialogCallback dialogCallback) {
		amount = amountSale;
		this.dialogCallback = dialogCallback;
		this.inputTypeText = inputTypeText;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setCancelable(true);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM,
				WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM);
		LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View rootView = inflater.inflate(R.layout.dialog_input_account, null);
		addContentView(rootView, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));
		inputType = (TextView) rootView.findViewById(R.id.prompt_tv);
		inputType.setText(inputTypeText);
		cardNumEdt = (EditText) rootView.findViewById(R.id.cardNum_edt);
		TextView amountTv = (TextView) rootView.findViewById(R.id.amount_tv);
		Button confirmBtn = (Button) rootView.findViewById(R.id.btConfirm);
		confirmBtn.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				String content = cardNumEdt.getText().toString().replace(" ", "");
				if (content == null || content.length() == 0) {
					Toast.makeText(getContext(), "Please swipe card or enter account", Toast.LENGTH_LONG).show();
					cardNumEdt.setText("");
					return;
				}

				String cardNum = cardNumEdt.getText().toString();
				cardNum = cardNum.replace(" ", "");
				dialogCallback.onConfirmCardNum(cardNum);
			}
		});
		Button cancelButton = (Button) rootView.findViewById(R.id.btCancel);
		cancelButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				AndroidTerminal.getInstance().cancelTransaction();
				dialogCallback.onCancelButtonPressed();
			}
		});
		amountTv.setText(amount);
	}

	@Override
	public void onWindowFocusChanged(boolean hasFocus) {
		super.onWindowFocusChanged(hasFocus);
		if (hasFocus) {
			getWindow().clearFlags(WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM);
		}
	}

	public interface DialogCallback {
		void onConfirmCardNum(String cardNum);
		void onCancelButtonPressed();
	}
}
