package com.payments.core.sdktest.common;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.text.InputType;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.payments.core.CoreSale;
import com.payments.core.admin.AndroidTerminal;
import com.payments.core.admin.TerminalBase;
import com.payments.core.admin.TerminalHealth;
import com.payments.core.sdktest.R;
import com.payments.core.sdktest.activities.PreferenceFragment;
import com.payments.core.sdktest.activities.SettingsFragment;

import java.math.BigDecimal;

import cn.pedant.SweetAlert.SweetAlertDialog;


public class UiHelper {
	private static UiHelper singleton = null;
	private SweetAlertDialog progressDialog;
	private EditText editTextExpiryDate;

	public static UiHelper getInstance() {
		if (singleton == null) {
			singleton = new UiHelper();
		}
		return singleton;
	}

	public void showError(Context context, String title, String message) {
		dismissProgress();
		new SweetAlertDialog(context, SweetAlertDialog.ERROR_TYPE)
				.setTitleText(title)
				.setContentText(message)
				.setConfirmClickListener(new SweetAlertDialog.OnSweetClickListener() {
					@Override
					public void onClick(SweetAlertDialog sDialog) {
						sDialog.dismissWithAnimation();
					}
				})
				.show();
	}

	public void showWarning(Context context, String title, String message) {
		dismissProgress();
		new SweetAlertDialog(context, SweetAlertDialog.WARNING_TYPE)
				.setTitleText(title)
				.setContentText(message)
				.setConfirmClickListener(new SweetAlertDialog.OnSweetClickListener() {
					@Override
					public void onClick(SweetAlertDialog sDialog) {
						sDialog.dismissWithAnimation();
					}
				})
				.show();
	}

	public void showSuccess(Context context, String title, String message) {
		dismissProgress();
		new SweetAlertDialog(context, SweetAlertDialog.SUCCESS_TYPE)
				.setTitleText(title)
				.setContentText(message)
				.setConfirmClickListener(new SweetAlertDialog.OnSweetClickListener() {
					@Override
					public void onClick(SweetAlertDialog sDialog) {
						sDialog.dismissWithAnimation();
					}
				})
				.show();
	}

	public void settingsMissing(final Context context, final SettingsFragment settingsActivity, final android.support.v4.app.FragmentManager manager) {
		dismissProgress();
		new SweetAlertDialog(context, SweetAlertDialog.WARNING_TYPE)
				.setTitleText("Settings missing")
				.setContentText("Terminal ID or secret are incorrect. Would you like to open application settings ?")
				.setConfirmClickListener(new SweetAlertDialog.OnSweetClickListener() {
					@Override
					public void onClick(SweetAlertDialog sDialog) {
						sDialog.dismissWithAnimation();
						manager.beginTransaction().replace(R.id.content_frame, settingsActivity).commit();
					}
				})
				.show();
	}

	public void showDialog(String message, Context context, final boolean isSale) {
		dismissProgress();
		progressDialog = new SweetAlertDialog(context, SweetAlertDialog.PROGRESS_TYPE);
		progressDialog.getProgressHelper().setBarColor(Color.parseColor("#c11d2c"));
		progressDialog.setTitleText(message);
		progressDialog.setCanceledOnTouchOutside(!isSale);
		progressDialog.setCancelable(!isSale);
		progressDialog.setCancelClickListener(new SweetAlertDialog.OnSweetClickListener() {
			@Override
			public void onClick(SweetAlertDialog sweetAlertDialog) {
				if (progressDialog != null) {
					progressDialog.dismissWithAnimation();
				}
				if (isSale) {
					AndroidTerminal.getInstance().cancelTransaction();
				}
			}
		});
		progressDialog.show();
	}

	public void updateMessage(String message) {
		if (progressDialog != null) {
			progressDialog.setTitleText(message);
		}
	}

	public boolean isProgressShowing() {
		return progressDialog != null && progressDialog.isShowing();
	}

	public void dismissProgress() {
		if (progressDialog != null) {
			progressDialog.dismissWithAnimation();
			progressDialog = null;
		}
	}

	public void showRequestAmountEdit(final CoreSale coreSale, Context context) {
		dismissProgress();
		final EditText taskEditText = new EditText(context);
		taskEditText.setRawInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
		taskEditText.setImeOptions(EditorInfo.IME_ACTION_DONE);
		taskEditText.addTextChangedListener(Util.addCurrencyMask(AndroidTerminal.getInstance().getSettings().getCurrency(), null, taskEditText));
		final AlertDialog dialog = new AlertDialog.Builder(context)
				.setTitle("Amount")
				.setMessage("Please enter amount")
				.setView(taskEditText)
				.setPositiveButton("Submit", null)
				.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						AndroidTerminal.getInstance().cancelTransaction();
					}
				})
				.create();

		dialog.setOnShowListener(new DialogInterface.OnShowListener() {
			@Override
			public void onShow(DialogInterface dialogInterface) {
				Button button = dialog.getButton(AlertDialog.BUTTON_POSITIVE);
				button.setOnClickListener(new View.OnClickListener() {
					@Override
					public void onClick(View view) {
						if (!TextUtils.isEmpty(taskEditText.getText())) {
							coreSale.setAmount(BigDecimal.valueOf(Double.parseDouble(taskEditText.getText().toString().replaceAll("[^0-9.]", ""))));
							AndroidTerminal.getInstance().submitAmount(coreSale, false);

							dialog.dismiss();
						} else {
							taskEditText.setError("Amount cannot be empty!");
						}
					}
				});
			}
		});
		dialog.show();
	}

	public void showOflineMode(Context context) {
		dismissProgress();
		AlertDialog dialog = new AlertDialog.Builder(context)
				.setTitle("Offline mode")
				.setMessage("Enable offline mode")
				.setPositiveButton("Enable", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						AndroidTerminal.getInstance().setOfflineMode(true);
					}
				})
				.setNegativeButton("Disable", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						AndroidTerminal.getInstance().setOfflineMode(false);
					}
				})
				.create();
		dialog.show();
	}

	public void showHealthCheck(Context context, final TerminalBase.TerminalHealthListener listener) {
		dismissProgress();
		AlertDialog dialog = new AlertDialog.Builder(context)
				.setTitle("Health Check")
				.setMessage("Enable Health Check")
				.setPositiveButton("Enable", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						TerminalHealth.startTerminalHealthCheck(AndroidTerminal.getInstance(), listener);
					}
				})
				.setNegativeButton("Disable", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						TerminalHealth.stopTerminalHealthCheck(AndroidTerminal.getInstance(), listener);
					}
				})
				.create();
		dialog.show();
	}

	public void showCloseBatch(Context context, final PreferenceFragment.CloseBatchListener listener) {
		dismissProgress();
		AlertDialog dialog = new AlertDialog.Builder(context)
				.setTitle("Request Close Batch")
				.setMessage("Request Close Batch")
				.setPositiveButton("Request", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						listener.onRequestCloseBatch();
					}
				})
				.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {

					}
				})
				.create();
		dialog.show();
	}

	public void showSignatureConfirmation(Context context, final DialogInterface.OnClickListener positiveAction, DialogInterface.OnClickListener negativeAction) {
		dismissProgress();
		AlertDialog dialog = new AlertDialog.Builder(context)
				.setTitle(context.getString(R.string.signature_confirmation_dialog_title))
				.setMessage(context.getString(R.string.signature_confirmation_dialog_message))
				.setPositiveButton(context.getString(R.string.signature_confirmation_dialog_yes), positiveAction)
				.setNegativeButton(context.getString(R.string.signature_confirmation_dialog_no), negativeAction)
				.create();
		dialog.show();
	}

	public void showConfirmation(Context context, final DialogInterface.OnClickListener positiveAction, DialogInterface.OnClickListener negativeAction, String title, String message) {
		dismissProgress();
		AlertDialog dialog = new AlertDialog.Builder(context)
				.setTitle(title)
				.setMessage(message)
				.setPositiveButton(context.getString(R.string.signature_confirmation_dialog_yes), positiveAction)
				.setNegativeButton(context.getString(R.string.signature_confirmation_dialog_no), negativeAction)
				.create();
		dialog.show();
	}

	public AlertDialog showPINDialog(Context context, String title, int inputType, String hint, final PromptDialogCallback callback) {
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		final EditText editText = new EditText(context);
		editText.setSelectAllOnFocus(true);
		editText.setInputType(inputType);
		editText.setHint(hint);
		editText.setHintTextColor(Color.parseColor("#77808080"));
		builder.setTitle(title);
		builder.setCancelable(true);
		builder.setView(editText);
		final AlertDialog dialog = builder.create();
		editText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
			@Override
			public void onFocusChange(View view, boolean focused) {
				if (focused) {
					dialog.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
				}
			}
		});
		callback.onStart(editText);
		return dialog;
	}

	public interface PromptDialogCallback {
		void onStart(EditText editText);
	}

	public void showExpiryDateDialog(Context context, final PromptDialogCallback callback) {
		editTextExpiryDate = new EditText(context);
		editTextExpiryDate.setHint("MMYY");
		editTextExpiryDate.setInputType(InputType.TYPE_CLASS_NUMBER);
		editTextExpiryDate.setSingleLine();
		editTextExpiryDate.setImeOptions(EditorInfo.IME_ACTION_DONE);
		final AlertDialog dialog = new AlertDialog.Builder(context).setTitle("Expiry date")
				.setMessage("Enter Expiry date!")
				.setView(editTextExpiryDate)
				.setPositiveButton("Confirm", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						callback.onStart(editTextExpiryDate);
					}
				})
				.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				})
				.show();

		editTextExpiryDate.setOnEditorActionListener(new TextView.OnEditorActionListener() {
			@Override
			public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
				if (actionId == EditorInfo.IME_ACTION_DONE || actionId == EditorInfo.IME_ACTION_UNSPECIFIED) {
					callback.onStart(editTextExpiryDate);
					dialog.dismiss();
					return true;
				}
				return false;
			}
		});

	}

	public String getExpiryDate() {
		if (editTextExpiryDate != null) {
			return editTextExpiryDate.getText().toString();
		}
		return "";
	}
}
