package com.payments.core.sdktest.common;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.widget.EditText;

import com.payments.core.common.enums.Currency;

import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class Util {

	public static String getCurrentTime() {
		DateFormat df = new SimpleDateFormat("dd/MM/yy HH:mm:ss", Locale.US);
		Date dateobj = new Date();
		return df.format(dateobj);
	}

	public static void saveToSettings(Context context, String key, String value) {
		SharedPreferences prefs = context.getSharedPreferences("prefs", Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = prefs.edit();
		editor.putString(key, value);
		editor.apply();
	}

	public static String getFromSettings(Context context, String key) {
		SharedPreferences prefs = context.getSharedPreferences("prefs", Context.MODE_PRIVATE);
		return prefs.getString(key, "");
	}

	public static String changeDateFormat(String dateString) {
		try{
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
			Date d = sdf.parse(dateString);
			sdf.applyPattern("yyyy-MM-dd HH:mm:ss");
			return sdf.format(d);
		} catch (Exception e){

		}
		return dateString;
	}

	public static TextWatcher addCurrencyMask(final Currency currency, final OnAmountChangeListener listener, final EditText tvAmount){
		return new TextWatcher() {
			private boolean mEditing = false;

			@Override
			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

			}

			@Override
			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

			}

			@Override
			public synchronized void afterTextChanged(Editable editable) {
				if(!mEditing){
					mEditing = true;

					String cleanString = editable.toString().replaceAll("\\D", "");
					if(!cleanString.isEmpty() && Double.parseDouble(cleanString) != 0){
						try{
							double number;

							if(currency.getExponent() == 2){
								number = Double.parseDouble(cleanString)/100;

							}else if(currency.getExponent() == 3){
								number = Double.parseDouble(cleanString)/1000;

							}else{
								number = Double.parseDouble(cleanString);
							}

							String formatted = currency.getSymbol() + getFormattedAmount(number, currency);

							tvAmount.setText(formatted);
							tvAmount.setSelection(tvAmount.getText().length());
							if(listener != null){
								listener.onAmountUpdated(number);
							}
						}catch (Exception error){
							editable.clear();
						}
					} else {
						editable.replace(0, editable.length(), "");
						if(listener != null){
							listener.onAmountUpdated(0);
						}
					}

					mEditing = false;
				}
			}
		};
	}

	public static String getFormattedAmount(double amount, Currency currency){
		try {
			NumberFormat numberFormat = NumberFormat.getInstance();
			numberFormat.setMaximumFractionDigits(currency.getExponent());
			numberFormat.setMinimumFractionDigits(currency.getExponent());
			numberFormat.setCurrency(java.util.Currency.getInstance(currency.value()));

			return numberFormat.format(amount);

		}catch (Exception error){
			return String.valueOf(amount);
		}
	}

	public interface OnAmountChangeListener{
		void onAmountUpdated(double newValue);
	}


	public static TextWatcher addCardNumberMask(){
		return new TextWatcher() {
			private static final char cardNumberSeparator = ' ';

			@Override
			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

			}

			@Override
			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

			}

			@Override
			public void afterTextChanged(Editable editable) {
				// Remove spacing char
				if (editable.length() > 0 && (editable.length() % 5) == 0) {
					final char ch = editable.charAt(editable.length() - 1);
					if (cardNumberSeparator == ch) {
						editable.delete(editable.length() - 1, editable.length());
					}
				}
				// Insert char where needed.
				if (editable.length() > 0 && (editable.length() % 5) == 0) {
					char ch = editable.charAt(editable.length() - 1);
					// Only if its a digit where there should be a space we insert a space
					if (Character.isDigit(ch) && TextUtils.split(editable.toString(), String.valueOf(cardNumberSeparator)).length <= 3) {
						editable.insert(editable.length() - 1, String.valueOf(cardNumberSeparator));
					}
				}
			}
		};
	}
}
