package com.payments.core.sdktest.view;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import com.payments.core.CoreSignature;
import com.payments.core.admin.AndroidTerminal;

/**
 * Canvas class that demonstrates how to draw the signature
 */
public class CanvasView extends View {
	private Path mPath;
	private Paint mPaint;
	private float mX, mY;
	private static final float TOLERANCE = 5;
	private CoreSignature signature;

	public CanvasView(Context c, AttributeSet attrs) {
		super(c, attrs);
		mPath = new Path();
		mPaint = new Paint();
		mPaint.setAntiAlias(true);
		mPaint.setColor(Color.BLACK);
		mPaint.setStyle(Paint.Style.STROKE);
		mPaint.setStrokeJoin(Paint.Join.ROUND);
		mPaint.setStrokeWidth(4f);
	}

	public void setSignature(CoreSignature signature) {
		this.signature = signature;
	}

	public CoreSignature getSignature() {
		return signature;
	}

	public void clearCanvas() {
		mPath.reset();
		invalidate();
	}

	public void submitSignature(AndroidTerminal terminal) {
		terminal.submitSignature(signature);
	}
	public boolean checkSignature(){
		return signature.checkSignature();
	}

	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		super.onSizeChanged(w, h, oldw, oldh);
	}

	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		// draw the mPath with the mPaint on the canvas when onDraw
		canvas.drawPath(mPath, mPaint);
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		float x = event.getX();
		float y = event.getY();

		switch (event.getAction()) {
			case MotionEvent.ACTION_DOWN:
				signature.startTouch(x, y);
				mPath.moveTo(x, y);
				mX = x;
				mY = y;
				invalidate();
				break;
			case MotionEvent.ACTION_MOVE:
				signature.moveTouch(x, y);

				float dx = Math.abs(x - mX);
				float dy = Math.abs(y - mY);
				if (dx >= TOLERANCE || dy >= TOLERANCE) {
					mPath.quadTo(mX, mY, (x + mX) / 2, (y + mY) / 2);
					mX = x;
					mY = y;
				}
				invalidate();
				break;
			case MotionEvent.ACTION_UP:
				signature.upTouch();
				mPath.lineTo(mX, mY);
				invalidate();
				break;
		}
		return true;
	}
}
